@@ -1,24 +1,41 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright (C) 2006 Cluster File Systems, Inc.
- *   Author: Eric Barton <eric@bartonsoftware.com>
+ * GPL HEADER START
  *
- *   This file is part of Lustre, http://www.lustre.org.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   Lustre is free software; you can redistribute it and/or
- *   modify it under the terms of version 2 of the GNU General Public
- *   License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   Lustre is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   You should have received a copy of the GNU General Public License
- *   along with Lustre; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
  *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lnet/klnds/o2iblnd/o2iblnd_modparams.c
+ *
+ * Author: Eric Barton <eric@bartonsoftware.com>
  */
 
 #include "o2iblnd.h"
@@ -113,9 +130,48 @@ kib_tunables_t kiblnd_tunables = {
 
 static char ipif_basename_space[32];
 
+#ifndef HAVE_SYSCTL_UNNUMBERED
+
+enum {
+        O2IBLND_SERVICE  = 1,
+        O2IBLND_CKSUM,
+        O2IBLND_TIMEOUT,
+        O2IBLND_NTX,
+        O2IBLND_CREDITS,
+        O2IBLND_PEER_CREDITS,
+        O2IBLND_IPIF_BASENAME,
+        O2IBLND_RETRY_COUNT,
+        O2IBLND_RNR_RETRY_COUNT,
+        O2IBLND_KEEPALIVE,
+        O2IBLND_CONCURRENT_SENDS,
+        O2IBLND_IB_MTU,
+        O2IBLND_FMR_POOL_SIZE,
+        O2IBLND_FMR_FLUSH_TRIGGER,
+        O2IBLND_FMR_CACHE
+};
+#else
+
+#define O2IBLND_SERVICE          CTL_UNNUMBERED
+#define O2IBLND_CKSUM            CTL_UNNUMBERED
+#define O2IBLND_TIMEOUT          CTL_UNNUMBERED
+#define O2IBLND_NTX              CTL_UNNUMBERED
+#define O2IBLND_CREDITS          CTL_UNNUMBERED
+#define O2IBLND_PEER_CREDITS     CTL_UNNUMBERED
+#define O2IBLND_IPIF_BASENAME    CTL_UNNUMBERED
+#define O2IBLND_RETRY_COUNT      CTL_UNNUMBERED
+#define O2IBLND_RNR_RETRY_COUNT  CTL_UNNUMBERED
+#define O2IBLND_KEEPALIVE        CTL_UNNUMBERED
+#define O2IBLND_CONCURRENT_SENDS CTL_UNNUMBERED
+#define O2IBLND_IB_MTU           CTL_UNNUMBERED
+#define O2IBLND_FMR_POOL_SIZE    CTL_UNNUMBERED
+#define O2IBLND_FMR_FLUSH_TRIGGER CTL_UNNUMBERED
+#define O2IBLND_FMR_CACHE        CTL_UNNUMBERED
+
+#endif
+
 static cfs_sysctl_table_t kiblnd_ctl_table[] = {
         {
-                .ctl_name = 1,
+                .ctl_name = O2IBLND_SERVICE,
                 .procname = "service",
                 .data     = &service,
                 .maxlen   = sizeof(int),
@@ -123,7 +179,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 2,
+                .ctl_name = O2IBLND_CKSUM,
                 .procname = "cksum",
                 .data     = &cksum,
                 .maxlen   = sizeof(int),
@@ -131,7 +187,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 3,
+                .ctl_name = O2IBLND_TIMEOUT,
                 .procname = "timeout",
                 .data     = &timeout,
                 .maxlen   = sizeof(int),
@@ -139,7 +195,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 4,
+                .ctl_name = O2IBLND_NTX,
                 .procname = "ntx",
                 .data     = &ntx,
                 .maxlen   = sizeof(int),
@@ -147,7 +203,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 5,
+                .ctl_name = O2IBLND_CREDITS,
                 .procname = "credits",
                 .data     = &credits,
                 .maxlen   = sizeof(int),
@@ -155,7 +211,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 6,
+                .ctl_name = O2IBLND_PEER_CREDITS,
                 .procname = "peer_credits",
                 .data     = &peer_credits,
                 .maxlen   = sizeof(int),
@@ -163,7 +219,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 7,
+                .ctl_name = O2IBLND_IPIF_BASENAME,
                 .procname = "ipif_name",
                 .data     = ipif_basename_space,
                 .maxlen   = sizeof(ipif_basename_space),
@@ -171,7 +227,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dostring
         },
         {
-                .ctl_name = 8,
+                .ctl_name = O2IBLND_RETRY_COUNT,
                 .procname = "retry_count",
                 .data     = &retry_count,
                 .maxlen   = sizeof(int),
@@ -179,7 +235,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 9,
+                .ctl_name = O2IBLND_RNR_RETRY_COUNT,
                 .procname = "rnr_retry_count",
                 .data     = &rnr_retry_count,
                 .maxlen   = sizeof(int),
@@ -187,7 +243,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 10,
+                .ctl_name = O2IBLND_KEEPALIVE,
                 .procname = "keepalive",
                 .data     = &keepalive,
                 .maxlen   = sizeof(int),
@@ -195,15 +251,15 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 11,
+                .ctl_name = O2IBLND_CONCURRENT_SENDS,
                 .procname = "concurrent_sends",
                 .data     = &concurrent_sends,
                 .maxlen   = sizeof(int),
-                .mode     = 0644,
+                .mode     = 0444,
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 12,
+                .ctl_name = O2IBLND_IB_MTU,
                 .procname = "ib_mtu",
                 .data     = &ib_mtu,
                 .maxlen   = sizeof(int),
@@ -212,7 +268,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
         },
 #if IBLND_MAP_ON_DEMAND
         {
-                .ctl_name = 13,
+                .ctl_name = O2IBLND_FMR_POOL_SIZE,
                 .procname = "fmr_pool_size",
                 .data     = &fmr_pool_size,
                 .maxlen   = sizeof(int),
@@ -220,7 +276,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 14,
+                .ctl_name = O2IBLND_FMR_FLUSH_TRIGGER,
                 .procname = "fmr_flush_trigger",
                 .data     = &fmr_flush_trigger,
                 .maxlen   = sizeof(int),
@@ -228,7 +284,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
                 .proc_handler = &proc_dointvec
         },
         {
-                .ctl_name = 15,
+                .ctl_name = O2IBLND_FMR_CACHE,
                 .procname = "fmr_cache",
                 .data     = &fmr_cache,
                 .maxlen   = sizeof(int),
@@ -241,7 +297,7 @@ static cfs_sysctl_table_t kiblnd_ctl_table[] = {
 
 static cfs_sysctl_table_t kiblnd_top_ctl_table[] = {
         {
-                .ctl_name = 203,
+                .ctl_name = CTL_O2IBLND,
                 .procname = "o2iblnd",
                 .data     = NULL,
                 .maxlen   = 0,
@@ -299,8 +355,14 @@ kiblnd_tunables_init (void)
 
         if (*kiblnd_tunables.kib_concurrent_sends > IBLND_RX_MSGS)
                 *kiblnd_tunables.kib_concurrent_sends = IBLND_RX_MSGS;
-        if (*kiblnd_tunables.kib_concurrent_sends < IBLND_MSG_QUEUE_SIZE)
-                *kiblnd_tunables.kib_concurrent_sends = IBLND_MSG_QUEUE_SIZE;
+        if (*kiblnd_tunables.kib_concurrent_sends < IBLND_MSG_QUEUE_SIZE / 2)
+                *kiblnd_tunables.kib_concurrent_sends = IBLND_MSG_QUEUE_SIZE / 2;
+
+        if (*kiblnd_tunables.kib_concurrent_sends < IBLND_MSG_QUEUE_SIZE) {
+                CWARN("Concurrent sends %d is lower than message queue size: %d, "
+                      "performance may drop slightly.\n",
+                      *kiblnd_tunables.kib_concurrent_sends, IBLND_MSG_QUEUE_SIZE);
+        }
 
         return 0;
 }
@@ -310,6 +372,3 @@ kiblnd_tunables_fini (void)
 {
         kiblnd_sysctl_fini();
 }
-
-
-
