@@ -1,19 +1,41 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- * Copyright (C) 2005 Cluster File Systems, Inc. All rights reserved.
- *   Author: Eric Barton <eeb@bartonsoftware.com>
+ * GPL HEADER START
  *
- *   This file is part of the Lustre file system, http://www.lustre.org
- *   Lustre is a trademark of Cluster File Systems, Inc.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   This file is confidential source code owned by Cluster File Systems.
- *   No viewing, modification, compilation, redistribution, or any other
- *   form of use is permitted except through a signed license agreement.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   If you have not signed such an agreement, then you have no rights to
- *   this file.  Please destroy it immediately and contact CFS.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lnet/ulnds/ptllnd/ptllnd.h
+ *
+ * Author: Eric Barton <eeb@bartonsoftware.com>
  */
 
 
@@ -118,11 +140,13 @@ typedef struct
 
         int                        plp_max_msg_size;
         int                        plp_refcount;
+        int                        plp_sent_hello:1;
         int                        plp_recvd_hello:1;
         int                        plp_closing:1;
         __u64                      plp_match;
         __u64                      plp_stamp;
         struct list_head           plp_txq;
+        struct list_head           plp_noopq;
         struct list_head           plp_activeq;
 } ptllnd_peer_t;
 
@@ -249,13 +273,13 @@ ptllnd_peer_decref (ptllnd_peer_t *peer)
 static inline lnet_nid_t
 ptllnd_ptl2lnetnid(lnet_ni_t *ni, ptl_nid_t portals_nid)
 {
-	return LNET_MKNID(LNET_NIDNET(ni->ni_nid), portals_nid);
+        return LNET_MKNID(LNET_NIDNET(ni->ni_nid), portals_nid);
 }
 
 static inline ptl_nid_t
 ptllnd_lnet2ptlnid(lnet_nid_t lnet_nid)
 {
-	return LNET_NIDADDR(lnet_nid);
+        return LNET_NIDADDR(lnet_nid);
 }
 
 /*
@@ -270,5 +294,3 @@ ptllnd_lnet2ptlnid(lnet_nid_t lnet_nid)
  *  boot the catamount node.  This works for debugging some simple
  *  cases.
  */
-
-
