@@ -503,6 +503,11 @@ static int fsfilt_ext3_setattr(struct dentry *dentry, void *handle,
         struct inode *inode = dentry->d_inode;
         int rc = 0;
 
+        LASSERTF(!(iattr->ia_valid & ATTR_SIZE) || !S_ISDIR(inode->i_mode),
+                 "changing i_size on directory #%llu (%p) new %llu old %llu\n",
+                 (long long)inode->i_ino, inode, (long long)iattr->ia_size,
+                 (long long)i_size_read(inode));
+
         /* Avoid marking the inode dirty on the superblock list unnecessarily.
          * We are already writing the inode to disk as part of this
          * transaction and want to avoid a lot of extra inode writeout
