@@ -1,10 +1,42 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- * Lustre administrative quota format.
+ * GPL HEADER START
  *
- *  from
- *  linux/fs/quota_v2.c
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
+ *
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
+ *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lustre/lvfs/lustre_quota_fmt.c
+ *
+ * Lustre administrative quota format.
+ * from linux/fs/quota_v2.c
  */
 
 #ifndef EXPORT_SYMTAB
@@ -27,6 +59,8 @@
 #include <obd_support.h>
 #include "lustre_quota_fmt.h"
 
+#ifdef HAVE_QUOTA_SUPPORT
+
 static const uint lustre_initqversions[][MAXQUOTAS] = {
         [LUSTRE_QUOTA_V1] = LUSTRE_INITQVERSIONS,
         [LUSTRE_QUOTA_V2] = LUSTRE_INITQVERSIONS_V2
@@ -138,9 +172,8 @@ int lustre_write_quota_info(struct lustre_quota_info *lqi, int type)
                               sizeof(struct lustre_disk_dqinfo), &offset);
         set_fs(fs);
         if (size != sizeof(struct lustre_disk_dqinfo)) {
-                printk(KERN_WARNING
-                       "Can't write info structure on device %s.\n",
-                       f->f_vfsmnt->mnt_sb->s_id);
+                CWARN("Can't write info structure on device %s.\n",
+                      f->f_vfsmnt->mnt_sb->s_id);
                 return -1;
         }
         return 0;
@@ -208,8 +241,7 @@ dqbuf_t getdqbuf(void)
 {
         dqbuf_t buf = kmalloc(LUSTRE_DQBLKSIZE, GFP_NOFS);
         if (!buf)
-                printk(KERN_WARNING
-                       "VFS: Not enough memory for quota buffers.\n");
+                CWARN("VFS: Not enough memory for quota buffers.\n");
         return buf;
 }
 
@@ -554,8 +586,8 @@ static int lustre_write_dquot(struct lustre_dquot *dquot,
                                 dqblk_sz, &offset);
         set_fs(fs);
         if (ret != dqblk_sz) {
-                printk(KERN_WARNING "VFS: dquota write failed on dev %s\n",
-                       filp->f_dentry->d_sb->s_id);
+                CWARN("VFS: dquota write failed on dev %s\n",
+                      filp->f_dentry->d_sb->s_id);
                 if (ret >= 0)
                         ret = -ENOSPC;
         } else
@@ -1077,3 +1109,4 @@ EXPORT_SYMBOL(lustre_read_dquot);
 EXPORT_SYMBOL(lustre_commit_dquot);
 EXPORT_SYMBOL(lustre_init_quota_info);
 EXPORT_SYMBOL(lustre_get_qids);
+#endif
