@@ -1,29 +1,43 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  mds/mds_fs.c
- *  Lustre Metadata Server (MDS) filesystem interface code
+ * GPL HEADER START
  *
- *  Copyright (C) 2002, 2003 Cluster File Systems, Inc.
- *   Author: Andreas Dilger <adilger@clusterfs.com>
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   This file is part of the Lustre file system, http://www.lustre.org
- *   Lustre is a trademark of Cluster File Systems, Inc.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   You may have signed or agreed to another license before downloading
- *   this software.  If so, you are bound by the terms and conditions
- *   of that agreement, and the following does not apply to you.  See the
- *   LICENSE file included with this distribution for more information.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   If you did not agree to a different license, then this copy of Lustre
- *   is open source software; you can redistribute it and/or modify it
- *   under the terms of version 2 of the GNU General Public License as
- *   published by the Free Software Foundation.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
  *
- *   In either case, Lustre is distributed in the hope that it will be
- *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
- *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   license text for more details.
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lustre/mds/mds_fs.c
+ *
+ * Lustre Metadata Server (MDS) filesystem interface code
+ *
+ * Author: Andreas Dilger <adilger@clusterfs.com>
  */
 
 #ifndef EXPORT_SYMTAB
@@ -36,9 +50,7 @@
 #include <linux/version.h>
 #include <linux/sched.h>
 #include <lustre_quota.h>
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
 #include <linux/mount.h>
-#endif
 #include <lustre_mds.h>
 #include <obd_class.h>
 #include <obd_support.h>
@@ -50,17 +62,23 @@
 #include "mds_internal.h"
 
 
-static int mds_export_stats_init(struct obd_device *obd,
+int mds_export_stats_init(struct obd_device *obd,
                                  struct obd_export *exp,
-                                 lnet_nid_t client_nid)
-  {
-        int rc, num_stats, newnid;
+                                 void *localdata)
+{
+        lnet_nid_t *client_nid = localdata;
+        int rc, num_stats, newnid = 0;
 
         rc = lprocfs_exp_setup(exp, client_nid, &newnid);
-        if (rc)
+        if (rc) {
+                /* Mask error for already created
+                 * /proc entries */
+                if (rc == -EALREADY)
+                        rc = 0;
                 return rc;
+        }
 
-        if (client_nid && newnid) {
+        if (newnid) {
                 struct nid_stat *tmp = exp->exp_nid_stats;
                 LASSERT(tmp != NULL);
 
@@ -78,6 +96,20 @@ static int mds_export_stats_init(struct obd_device *obd,
                         return rc;
 
                 mds_stats_counter_init(tmp->nid_stats);
+
+                /* Always add in ldlm_stats */
+                tmp->nid_ldlm_stats = lprocfs_alloc_stats(LDLM_LAST_OPC -
+                                                          LDLM_FIRST_OPC,
+                                                          0);
+                if (tmp->nid_ldlm_stats == NULL)
+                        return -ENOMEM;
+
+                lprocfs_init_ldlm_stats(tmp->nid_ldlm_stats);
+
+                rc = lprocfs_register_stats(tmp->nid_proc, "ldlm_stats",
+                                            tmp->nid_ldlm_stats);
+                if (rc)
+                        return rc;
         }
 
         return 0;
@@ -92,7 +124,7 @@ static int mds_export_stats_init(struct obd_device *obd,
  * mds_init_server_data() callsite needs to be fixed.
  */
 int mds_client_add(struct obd_device *obd, struct obd_export *exp,
-                   int cl_idx, lnet_nid_t client_nid)
+                   int cl_idx, void *localdata)
 {
         struct mds_obd *mds = &obd->u.mds;
         struct mds_export_data *med = &exp->exp_mds_data;
@@ -104,8 +136,8 @@ int mds_client_add(struct obd_device *obd, struct obd_export *exp,
         LASSERT(bitmap != NULL);
         LASSERTF(cl_idx > -2, "%d\n", cl_idx);
 
-        /* XXX if mcd_uuid were a real obd_uuid, I could use obd_uuid_equals */
-        if (!strcmp(med->med_mcd->mcd_uuid, obd->obd_uuid.uuid))
+        /* XXX if lcd_uuid were a real obd_uuid, I could use obd_uuid_equals */
+        if (!strcmp(med->med_lcd->lcd_uuid, obd->obd_uuid.uuid))
                 RETURN(0);
 
         /* the bitmap operations can handle cl_idx > sizeof(long) * 8, so
@@ -134,21 +166,27 @@ int mds_client_add(struct obd_device *obd, struct obd_export *exp,
         }
 
         CDEBUG(D_INFO, "client at idx %d with UUID '%s' added\n",
-               cl_idx, med->med_mcd->mcd_uuid);
+               cl_idx, med->med_lcd->lcd_uuid);
 
         med->med_lr_idx = cl_idx;
         med->med_lr_off = le32_to_cpu(mds->mds_server_data->lsd_client_start) +
                 (cl_idx * le16_to_cpu(mds->mds_server_data->lsd_client_size));
         LASSERTF(med->med_lr_off > 0, "med_lr_off = %llu\n", med->med_lr_off);
-        mds_export_stats_init(obd, exp, client_nid);
+        mds_export_stats_init(obd, exp, localdata);
 
         if (new_client) {
-                struct lvfs_run_ctxt saved;
+                struct lvfs_run_ctxt *saved = NULL;
                 loff_t off = med->med_lr_off;
                 struct file *file = mds->mds_rcvd_filp;
                 void *handle;
 
-                push_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
+                OBD_SLAB_ALLOC_PTR(saved, obd_lvfs_ctxt_cache);
+                if (saved == NULL) {
+                        CERROR("cannot allocate memory for run ctxt\n");
+                        RETURN(-ENOMEM);
+                }
+
+                push_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
                 handle = fsfilt_start(obd, file->f_dentry->d_inode,
                                       FSFILT_OP_SETATTR, NULL);
                 if (IS_ERR(handle)) {
@@ -162,47 +200,48 @@ int mds_client_add(struct obd_device *obd, struct obd_export *exp,
                                 exp->exp_need_sync = 1;
                                 spin_unlock(&exp->exp_lock);
                         }
-                        rc = fsfilt_write_record(obd, file, med->med_mcd,
-                                                 sizeof(*med->med_mcd),
+                        rc = fsfilt_write_record(obd, file, med->med_lcd,
+                                                 sizeof(*med->med_lcd),
                                                  &off, rc /* sync if no cb */);
                         fsfilt_commit(obd, file->f_dentry->d_inode, handle, 0);
                 }
 
-                pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
+                pop_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
+                OBD_SLAB_FREE_PTR(saved, obd_lvfs_ctxt_cache);
 
                 if (rc)
                         return rc;
-                CDEBUG(D_INFO, "wrote client mcd at idx %u off %llu (len %u)\n",
+                CDEBUG(D_INFO, "wrote client lcd at idx %u off %llu (len %u)\n",
                        med->med_lr_idx, med->med_lr_off,
-                       (unsigned int)sizeof(*med->med_mcd));
+                       (unsigned int)sizeof(*med->med_lcd));
         }
         return 0;
 }
 
+struct lsd_client_data zero_lcd; /* globals are implicitly zeroed */
+
 int mds_client_free(struct obd_export *exp)
 {
         struct mds_export_data *med = &exp->exp_mds_data;
         struct mds_obd *mds = &exp->exp_obd->u.mds;
         struct obd_device *obd = exp->exp_obd;
-        struct mds_client_data zero_mcd;
-        struct lvfs_run_ctxt saved;
+        struct lvfs_run_ctxt *saved = NULL;
         int rc;
         loff_t off;
         ENTRY;
 
-        if (!med->med_mcd)
+        if (!med->med_lcd)
                 RETURN(0);
 
-        /* XXX if mcd_uuid were a real obd_uuid, I could use obd_uuid_equals */
-        if (!strcmp(med->med_mcd->mcd_uuid, obd->obd_uuid.uuid))
+        /* XXX if lcd_uuid were a real obd_uuid, I could use obd_uuid_equals */
+        if (!strcmp(med->med_lcd->lcd_uuid, obd->obd_uuid.uuid))
                 GOTO(free, 0);
 
         CDEBUG(D_INFO, "freeing client at idx %u, offset %lld with UUID '%s'\n",
-               med->med_lr_idx, med->med_lr_off, med->med_mcd->mcd_uuid);
+               med->med_lr_idx, med->med_lr_off, med->med_lcd->lcd_uuid);
 
         LASSERT(mds->mds_client_bitmap != NULL);
 
-        lprocfs_exp_cleanup(exp);
 
         off = med->med_lr_off;
 
@@ -223,17 +262,32 @@ int mds_client_free(struct obd_export *exp)
         }
 
         if (!(exp->exp_flags & OBD_OPT_FAILOVER)) {
-                memset(&zero_mcd, 0, sizeof zero_mcd);
-                push_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
-                rc = fsfilt_write_record(obd, mds->mds_rcvd_filp, &zero_mcd,
-                                         sizeof(zero_mcd), &off,
-                                         (!exp->exp_libclient ||
-                                          exp->exp_need_sync));
-                pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
+                /* Don't force sync on each disconnect if aborting recovery,
+                 * or it does num_clients * num_osts syncs.  b=17194 */
+                int need_sync = (!exp->exp_libclient || exp->exp_need_sync) &&
+                                 !(exp->exp_flags & OBD_OPT_ABORT_RECOV);
+
+                OBD_SLAB_ALLOC_PTR(saved, obd_lvfs_ctxt_cache);
+                if (saved == NULL) {
+                        CERROR("cannot allocate memory for run ctxt\n");
+                        GOTO(free, rc = -ENOMEM);
+                }
+                push_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
+                rc = fsfilt_write_record(obd, mds->mds_rcvd_filp, &zero_lcd,
+                                         sizeof(zero_lcd), &off, 0);
+
+                /* Make sure the server's last_transno is up to date. Do this
+                 * after the client is freed so we know all the client's
+                 * transactions have been committed. */
+                if (rc == 0)
+                        mds_update_server_data(exp->exp_obd, need_sync);
+
+                pop_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
 
                 CDEBUG(rc == 0 ? D_INFO : D_ERROR,
-                       "zeroing out client %s idx %u in %s rc %d\n",
-                       med->med_mcd->mcd_uuid, med->med_lr_idx, LAST_RCVD, rc);
+                       "zero out client %s at idx %u/%llu in %s %ssync rc %d\n",
+                       med->med_lcd->lcd_uuid, med->med_lr_idx, med->med_lr_off,
+                       LAST_RCVD, need_sync ? "" : "a", rc);
         }
 
         if (!test_and_clear_bit(med->med_lr_idx, mds->mds_client_bitmap)) {
@@ -242,16 +296,13 @@ int mds_client_free(struct obd_export *exp)
                 LBUG();
         }
 
-
-        /* Make sure the server's last_transno is up to date. Do this
-         * after the client is freed so we know all the client's
-         * transactions have been committed. */
-        mds_update_server_data(exp->exp_obd, 0);
-
         EXIT;
- free:
-        OBD_FREE(med->med_mcd, sizeof(*med->med_mcd));
-        med->med_mcd = NULL;
+free:
+        if (saved)
+                OBD_SLAB_FREE_PTR(saved, obd_lvfs_ctxt_cache);
+
+        OBD_FREE_PTR(med->med_lcd);
+        med->med_lcd = NULL;
 
         return 0;
 }
@@ -269,7 +320,7 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
 {
         struct mds_obd *mds = &obd->u.mds;
         struct lr_server_data *lsd;
-        struct mds_client_data *mcd = NULL;
+        struct lsd_client_data *lcd = NULL;
         loff_t off = 0;
         unsigned long last_rcvd_size = i_size_read(file->f_dentry->d_inode);
         __u64 mount_count;
@@ -279,8 +330,8 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
         /* ensure padding in the struct is the correct size */
         LASSERT(offsetof(struct lr_server_data, lsd_padding) +
                 sizeof(lsd->lsd_padding) == LR_SERVER_SIZE);
-        LASSERT(offsetof(struct mds_client_data, mcd_padding) +
-                sizeof(mcd->mcd_padding) == LR_CLIENT_SIZE);
+        LASSERT(offsetof(struct lsd_client_data, lcd_padding) +
+                sizeof(lcd->lcd_padding) == LR_CLIENT_SIZE);
 
         OBD_ALLOC_WAIT(lsd, sizeof(*lsd));
         if (!lsd)
@@ -377,59 +428,59 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
                 struct obd_export *exp;
                 struct mds_export_data *med;
 
-                if (!mcd) {
-                        OBD_ALLOC_WAIT(mcd, sizeof(*mcd));
-                        if (!mcd)
+                if (!lcd) {
+                        OBD_ALLOC_WAIT(lcd, sizeof(*lcd));
+                        if (!lcd)
                                 GOTO(err_client, rc = -ENOMEM);
                 }
 
                 /* Don't assume off is incremented properly by
-                 * fsfilt_read_record(), in case sizeof(*mcd)
+                 * fsfilt_read_record(), in case sizeof(*lcd)
                  * isn't the same as lsd->lsd_client_size.  */
                 off = le32_to_cpu(lsd->lsd_client_start) +
                         cl_idx * le16_to_cpu(lsd->lsd_client_size);
-                rc = fsfilt_read_record(obd, file, mcd, sizeof(*mcd), &off);
+                rc = fsfilt_read_record(obd, file, lcd, sizeof(*lcd), &off);
                 if (rc) {
                         CERROR("error reading MDS %s idx %d, off %llu: rc %d\n",
                                LAST_RCVD, cl_idx, off, rc);
                         break; /* read error shouldn't cause startup to fail */
                 }
 
-                if (mcd->mcd_uuid[0] == '\0') {
+                if (lcd->lcd_uuid[0] == '\0') {
                         CDEBUG(D_INFO, "skipping zeroed client at offset %d\n",
                                cl_idx);
                         continue;
                 }
 
-                last_transno = le64_to_cpu(mcd->mcd_last_transno) >
-                               le64_to_cpu(mcd->mcd_last_close_transno) ?
-                               le64_to_cpu(mcd->mcd_last_transno) :
-                               le64_to_cpu(mcd->mcd_last_close_transno);
+                last_transno = le64_to_cpu(lcd->lcd_last_transno) >
+                               le64_to_cpu(lcd->lcd_last_close_transno) ?
+                               le64_to_cpu(lcd->lcd_last_transno) :
+                               le64_to_cpu(lcd->lcd_last_close_transno);
 
                 /* These exports are cleaned up by mds_disconnect(), so they
                  * need to be set up like real exports as mds_connect() does.
                  */
                 CDEBUG(D_HA, "RCVRNG CLIENT uuid: %s idx: %d lr: "LPU64
-                       " srv lr: "LPU64" lx: "LPU64"\n", mcd->mcd_uuid, cl_idx,
+                       " srv lr: "LPU64" lx: "LPU64"\n", lcd->lcd_uuid, cl_idx,
                        last_transno, le64_to_cpu(lsd->lsd_last_transno),
-                       le64_to_cpu(mcd->mcd_last_xid));
+                       le64_to_cpu(lcd->lcd_last_xid));
 
-                exp = class_new_export(obd, (struct obd_uuid *)mcd->mcd_uuid);
+                exp = class_new_export(obd, (struct obd_uuid *)lcd->lcd_uuid);
                 if (IS_ERR(exp)) {
                         if (PTR_ERR(exp) == -EALREADY) {
                                 /* export already exists, zero out this one */
-                                mcd->mcd_uuid[0] = '\0';
+                                lcd->lcd_uuid[0] = '\0';
                         } else {
                                 GOTO(err_client, rc = PTR_ERR(exp));
                         }
                 } else {
                         med = &exp->exp_mds_data;
-                        med->med_mcd = mcd;
-                        rc = mds_client_add(obd, exp, cl_idx, 0);
+                        med->med_lcd = lcd;
+                        rc = mds_client_add(obd, exp, cl_idx, NULL);
                         /* can't fail for existing client */
                         LASSERTF(rc == 0, "rc = %d\n", rc);
 
-                        mcd = NULL;
+                        lcd = NULL;
 
                         spin_lock(&exp->exp_lock);
                         exp->exp_replay_needed = 1;
@@ -449,8 +500,8 @@ static int mds_init_server_data(struct obd_device *obd, struct file *file)
                         mds->mds_last_transno = last_transno;
         }
 
-        if (mcd)
-                OBD_FREE(mcd, sizeof(*mcd));
+        if (lcd)
+                OBD_FREE_PTR(lcd);
 
         obd->obd_last_committed = mds->mds_last_transno;
 
@@ -490,7 +541,7 @@ err_msd:
 int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
 {
         struct mds_obd *mds = &obd->u.mds;
-        struct lvfs_run_ctxt saved;
+        struct lvfs_run_ctxt *saved = NULL;
         struct dentry *dentry;
         struct file *file;
         int rc;
@@ -502,6 +553,12 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
         if (rc)
                 RETURN(rc);
 
+        OBD_SLAB_ALLOC_PTR(saved, obd_lvfs_ctxt_cache);
+        if (saved == NULL) {
+                CERROR("cannot allocate memory for run ctxt\n");
+                RETURN(-ENOMEM);
+        }
+
         mds->mds_vfsmnt = mnt;
         /* why not mnt->mnt_sb instead of mnt->mnt_root->d_inode->i_sb? */
         obd->u.obt.obt_sb = mnt->mnt_root->d_inode->i_sb;
@@ -517,8 +574,8 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
         obd->obd_lvfs_ctxt.cb_ops = mds_lvfs_ops;
 
         /* setup the directory tree */
-        push_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
-        dentry = simple_mkdir(current->fs->pwd, "ROOT", 0755, 0);
+        push_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
+        dentry = simple_mkdir(current->fs->pwd, mnt, "ROOT", 0755, 0);
         if (IS_ERR(dentry)) {
                 rc = PTR_ERR(dentry);
                 CERROR("cannot create ROOT directory: rc = %d\n", rc);
@@ -546,7 +603,7 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
                 GOTO(err_fid, rc);
         }
 
-        dentry = simple_mkdir(current->fs->pwd, "PENDING", 0777, 1);
+        dentry = simple_mkdir(current->fs->pwd, mnt, "PENDING", 0777, 1);
         if (IS_ERR(dentry)) {
                 rc = PTR_ERR(dentry);
                 CERROR("cannot create PENDING directory: rc = %d\n", rc);
@@ -555,7 +612,7 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
         mds->mds_pending_dir = dentry;
 
         /* COMPAT_146 */
-        dentry = simple_mkdir(current->fs->pwd, MDT_LOGS_DIR, 0777, 1);
+        dentry = simple_mkdir(current->fs->pwd, mnt, MDT_LOGS_DIR, 0777, 1);
         if (IS_ERR(dentry)) {
                 rc = PTR_ERR(dentry);
                 CERROR("cannot create %s directory: rc = %d\n",
@@ -565,7 +622,7 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
         mds->mds_logs_dir = dentry;
         /* end COMPAT_146 */
 
-        dentry = simple_mkdir(current->fs->pwd, "OBJECTS", 0777, 1);
+        dentry = simple_mkdir(current->fs->pwd, mnt, "OBJECTS", 0777, 1);
         if (IS_ERR(dentry)) {
                 rc = PTR_ERR(dentry);
                 CERROR("cannot create OBJECTS directory: rc = %d\n", rc);
@@ -616,8 +673,8 @@ int mds_fs_setup(struct obd_device *obd, struct vfsmount *mnt)
         if (rc)
                 GOTO(err_health_check, rc);
 err_pop:
-        pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
-
+        pop_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
+        OBD_SLAB_FREE_PTR(saved, obd_lvfs_ctxt_cache);
         return rc;
 
 err_health_check:
@@ -645,9 +702,15 @@ err_fid:
 int mds_fs_cleanup(struct obd_device *obd)
 {
         struct mds_obd *mds = &obd->u.mds;
-        struct lvfs_run_ctxt saved;
+        struct lvfs_run_ctxt *saved = NULL;
         int rc = 0;
 
+        OBD_SLAB_ALLOC_PTR(saved, obd_lvfs_ctxt_cache);
+        if (saved == NULL) {
+                CERROR("cannot allocate memory for run ctxt\n");
+                RETURN(-ENOMEM);
+        }
+
         if (obd->obd_fail)
                 LCONSOLE_WARN("%s: shutting down for failover; client state "
                               "will be preserved.\n", obd->obd_name);
@@ -655,7 +718,7 @@ int mds_fs_cleanup(struct obd_device *obd)
         class_disconnect_exports(obd); /* cleans up client info too */
         mds_server_free_data(mds);
 
-        push_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
+        push_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
         if (mds->mds_rcvd_filp) {
                 rc = filp_close(mds->mds_rcvd_filp, 0);
                 mds->mds_rcvd_filp = NULL;
@@ -686,7 +749,8 @@ int mds_fs_cleanup(struct obd_device *obd)
 
         lquota_fs_cleanup(mds_quota_interface_ref, obd);
 
-        pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
+        pop_ctxt(saved, &obd->obd_lvfs_ctxt, NULL);
+        OBD_SLAB_FREE_PTR(saved, obd_lvfs_ctxt_cache);
         shrink_dcache_parent(mds->mds_fid_de);
         dput(mds->mds_fid_de);
         LL_DQUOT_OFF(obd->u.obt.obt_sb);
@@ -705,17 +769,23 @@ int mds_obd_create(struct obd_export *exp, struct obdo *oa,
         unsigned int tmpname = ll_rand();
         struct file *filp;
         struct dentry *new_child;
-        struct lvfs_run_ctxt saved;
+        struct lvfs_run_ctxt *saved = NULL;
         char fidname[LL_FID_NAMELEN];
         void *handle;
         struct lvfs_ucred ucred = { 0 };
         int rc = 0, err, namelen;
         ENTRY;
 
+        OBD_SLAB_ALLOC_PTR(saved, obd_lvfs_ctxt_cache);
+        if (saved == NULL) {
+                CERROR("cannot allocate memory for run ctxt\n");
+                RETURN(-ENOMEM);
+        }
+
         /* the owner of object file should always be root */
         cap_raise(ucred.luc_cap, CAP_SYS_RESOURCE);
 
-        push_ctxt(&saved, &exp->exp_obd->obd_lvfs_ctxt, &ucred);
+        push_ctxt(saved, &exp->exp_obd->obd_lvfs_ctxt, &ucred);
 
         sprintf(fidname, "OBJECTS/%u.%u", tmpname, current->pid);
         filp = filp_open(fidname, O_CREAT | O_EXCL, 0666);
@@ -755,8 +825,9 @@ int mds_obd_create(struct obd_export *exp, struct obdo *oa,
                 GOTO(out_dput, rc = PTR_ERR(handle));
 
         lock_kernel();
-        rc = vfs_rename(mds->mds_objects_dir->d_inode, filp->f_dentry,
-                        mds->mds_objects_dir->d_inode, new_child);
+        rc = ll_vfs_rename(mds->mds_objects_dir->d_inode, filp->f_dentry,
+                           filp->f_vfsmnt, mds->mds_objects_dir->d_inode,
+                           new_child, filp->f_vfsmnt);
         unlock_kernel();
         if (rc)
                 CERROR("error renaming new object "LPU64":%u: rc %d\n",
@@ -779,7 +850,8 @@ out_close:
                         rc = err;
         }
 out_pop:
-        pop_ctxt(&saved, &exp->exp_obd->obd_lvfs_ctxt, &ucred);
+        pop_ctxt(saved, &exp->exp_obd->obd_lvfs_ctxt, &ucred);
+        OBD_SLAB_FREE_PTR(saved, obd_lvfs_ctxt_cache);
         RETURN(rc);
 }
 
@@ -790,7 +862,7 @@ int mds_obd_destroy(struct obd_export *exp, struct obdo *oa,
         struct mds_obd *mds = &exp->exp_obd->u.mds;
         struct inode *parent_inode = mds->mds_objects_dir->d_inode;
         struct obd_device *obd = exp->exp_obd;
-        struct lvfs_run_ctxt saved;
+        struct lvfs_run_ctxt *saved = NULL;
         struct lvfs_ucred ucred = { 0 };
         char fidname[LL_FID_NAMELEN];
         struct inode *inode = NULL;
@@ -799,8 +871,14 @@ int mds_obd_destroy(struct obd_export *exp, struct obdo *oa,
         int err, namelen, rc = 0;
         ENTRY;
 
+        OBD_SLAB_ALLOC_PTR(saved, obd_lvfs_ctxt_cache);
+        if (saved == NULL) {
+                CERROR("cannot allocate memory for run ctxt\n");
+                RETURN(-ENOMEM);
+        }
+
         cap_raise(ucred.luc_cap, CAP_SYS_RESOURCE);
-        push_ctxt(&saved, &obd->obd_lvfs_ctxt, &ucred);
+        push_ctxt(saved, &obd->obd_lvfs_ctxt, &ucred);
 
         namelen = ll_fid2str(fidname, oa->o_id, oa->o_generation);
 
@@ -831,7 +909,7 @@ int mds_obd_destroy(struct obd_export *exp, struct obdo *oa,
            vfs_unlink() context. bug 10409 */
         inode = de->d_inode;
         atomic_inc(&inode->i_count);
-        rc = vfs_unlink(mds->mds_objects_dir->d_inode, de);
+        rc = ll_vfs_unlink(mds->mds_objects_dir->d_inode, de, mds->mds_vfsmnt);
         if (rc)
                 CERROR("error destroying object "LPU64":%u: rc %d\n",
                        oa->o_id, oa->o_generation, rc);
@@ -847,6 +925,7 @@ out_dput:
         if (inode)
                 iput(inode);
 
-        pop_ctxt(&saved, &obd->obd_lvfs_ctxt, &ucred);
+        pop_ctxt(saved, &obd->obd_lvfs_ctxt, &ucred);
+        OBD_SLAB_FREE_PTR(saved, obd_lvfs_ctxt_cache);
         RETURN(rc);
 }
