@@ -528,10 +528,10 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
 
         /* copy rc, transno and disp; steal locks */
         mds_req_from_lcd(req, lcd);
-        intent_set_disposition(rep, le32_to_cpu(lcd->lcd_last_data));
+        ldlm_reply_set_disposition(rep, le32_to_cpu(lcd->lcd_last_data));
 
         /* Only replay if create or open actually happened. */
-        if (!intent_disposition(rep, DISP_OPEN_CREATE | DISP_OPEN_OPEN) ) {
+        if (!ldlm_reply_disposition(rep, DISP_OPEN_CREATE | DISP_OPEN_OPEN) ) {
                 EXIT;
                 return; /* error looking up parent or child */
         }
@@ -569,8 +569,8 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
         /* At this point, we know we have a child. We'll send
          * it back _unless_ it not created and open failed.
          */
-        if (intent_disposition(rep, DISP_OPEN_OPEN) &&
-            !intent_disposition(rep, DISP_OPEN_CREATE) &&
+        if (ldlm_reply_disposition(rep, DISP_OPEN_OPEN) &&
+            !ldlm_reply_disposition(rep, DISP_OPEN_CREATE) &&
             req->rq_status) {
                 GOTO(out_dput, 0);
         }
@@ -617,7 +617,7 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
         /* If we didn't get as far as trying to open, then some locking thing
          * probably went wrong, and we'll just bail here.
          */
-        if (!intent_disposition(rep, DISP_OPEN_OPEN))
+        if (!ldlm_reply_disposition(rep, DISP_OPEN_OPEN))
                 GOTO(out_dput, 0);
 
         /* If we failed, then we must have failed opening, so don't look for
@@ -676,14 +676,14 @@ static void reconstruct_open(struct mds_update_record *rec, int offset,
 
         mds_mfd_put(mfd);
 
-        if (!intent_disposition(rep, DISP_OPEN_LOCK))
+        if (!ldlm_reply_disposition(rep, DISP_OPEN_LOCK))
                 GOTO(out_dput, 0);
 
         /* child_lockh has been set in fixup_handle_for_resent_req called
          * in mds_intent_policy for resent request */
         if (child_lockh == NULL || !lustre_handle_is_used(child_lockh)) {
                 /* the lock is already canceled! clear DISP_OPEN_LOCK */
-                intent_disposition(rep, ~DISP_OPEN_LOCK);
+                ldlm_reply_clear_disposition(rep, DISP_OPEN_LOCK);
         }
 
  out_dput:
@@ -799,7 +799,7 @@ static int mds_finish_open(struct ptlrpc_request *req, struct dentry *dchild,
         if ((rc = mds_lov_prepare_objids(obd,lmm)) != 0)
                 RETURN(rc);
 
-        intent_set_disposition(rep, DISP_OPEN_OPEN);
+        ldlm_reply_set_disposition(rep, DISP_OPEN_OPEN);
         mfd = mds_dentry_open(dchild, mds->mds_vfsmnt, flags, req);
         if (IS_ERR(mfd))
                 RETURN(PTR_ERR(mfd));
@@ -851,8 +851,8 @@ static int mds_open_by_fid(struct ptlrpc_request *req, struct ll_fid *fid,
 
         mds_pack_inode2fid(&body->fid1, dchild->d_inode);
         mds_pack_inode2body(body, dchild->d_inode);
-        intent_set_disposition(rep, DISP_LOOKUP_EXECD);
-        intent_set_disposition(rep, DISP_LOOKUP_POS);
+        ldlm_reply_set_disposition(rep, DISP_LOOKUP_EXECD);
+        ldlm_reply_set_disposition(rep, DISP_LOOKUP_POS);
 
         rc = mds_finish_open(req, dchild, body, flags, &handle, rec, rep, NULL);
         rc = mds_finish_transno(mds, dchild->d_inode, handle,
@@ -936,7 +936,7 @@ int mds_open(struct mds_update_record *rec, int offset,
         /* Always returning LOOKUP lock if open succesful to guard
            dentry on client. */
         int lock_flags = 0;
-        int rec_pending = 0;
+        int quota_pending[2] = {0, 0};
         int use_parent, need_open_lock;
         unsigned int gid = current->fsgid;
         ENTRY;
@@ -1054,8 +1054,8 @@ int mds_open(struct mds_update_record *rec, int offset,
                 } else {
                         /* Just cannot find parent - make it look like
                          * usual negative lookup to avoid extra MDS RPC */
-                        intent_set_disposition(rep, DISP_LOOKUP_EXECD);
-                        intent_set_disposition(rep, DISP_LOOKUP_NEG);
+                        ldlm_reply_set_disposition(rep, DISP_LOOKUP_EXECD);
+                        ldlm_reply_set_disposition(rep, DISP_LOOKUP_NEG);
                 }
                 GOTO(cleanup, rc);
         }
@@ -1074,11 +1074,11 @@ int mds_open(struct mds_update_record *rec, int offset,
 
         cleanup_phase = 2; /* child dentry */
 
-        intent_set_disposition(rep, DISP_LOOKUP_EXECD);
+        ldlm_reply_set_disposition(rep, DISP_LOOKUP_EXECD);
         if (dchild->d_inode)
-                intent_set_disposition(rep, DISP_LOOKUP_POS);
+                ldlm_reply_set_disposition(rep, DISP_LOOKUP_POS);
         else
-                intent_set_disposition(rep, DISP_LOOKUP_NEG);
+                ldlm_reply_set_disposition(rep, DISP_LOOKUP_NEG);
 
         /*Step 3: If the child was negative, and we're supposed to, create it.*/
         if (dchild->d_inode == NULL) {
@@ -1104,10 +1104,10 @@ int mds_open(struct mds_update_record *rec, int offset,
                  * FIXME: after CMD is used, pointer to obd_trans_info* couldn't
                  * be NULL, b=14840 */
                 lquota_chkquota(mds_quota_interface_ref, obd,
-                                current->fsuid, gid, 1, &rec_pending,
+                                current->fsuid, gid, 1, quota_pending,
                                 NULL, NULL, 0);
 
-                intent_set_disposition(rep, DISP_OPEN_CREATE);
+                ldlm_reply_set_disposition(rep, DISP_OPEN_CREATE);
                 handle = fsfilt_start(obd, dparent->d_inode, FSFILT_OP_CREATE,
                                       NULL);
                 if (IS_ERR(handle)) {
@@ -1170,7 +1170,7 @@ int mds_open(struct mds_update_record *rec, int offset,
                 /* for nfs and join - we need two locks for same fid, but
                  * with different mode */
                 if (need_open_lock && !use_parent)  {
-                        intent_set_disposition(rep, DISP_OPEN_LOCK);
+                        ldlm_reply_set_disposition(rep, DISP_OPEN_LOCK);
                         need_open_lock = 0;
                 }
         }
@@ -1226,7 +1226,7 @@ found_child:
                         GOTO(cleanup, rc = -EACCES);
                 }
                 if (ll_permission(dchild->d_inode, acc_mode, NULL)) {
-                        intent_set_disposition(rep, DISP_OPEN_OPEN);
+                        ldlm_reply_set_disposition(rep, DISP_OPEN_OPEN);
                         GOTO(cleanup, rc = -EACCES);
                 }
         } else if (rec->ur_flags & MDS_OPEN_DIRECTORY) {
@@ -1256,7 +1256,7 @@ found_child:
                         GOTO(cleanup, rc);
 
                 /* Let mds_intent_policy know that we have a lock to return */
-                intent_set_disposition(rep, DISP_OPEN_LOCK);
+                ldlm_reply_set_disposition(rep, DISP_OPEN_LOCK);
         }
 
         if (!S_ISREG(dchild->d_inode->i_mode) &&
@@ -1277,9 +1277,9 @@ found_child:
                                 req, rc, rep ? rep->lock_policy_res1 : 0, 0);
 
  cleanup_no_trans:
-        if (rec_pending)
+        if (quota_pending[0] || quota_pending[1])
                 lquota_pending_commit(mds_quota_interface_ref, obd,
-                                      current->fsuid, gid, rec_pending);
+                                      current->fsuid, gid, quota_pending);
         switch (cleanup_phase) {
         case 2:
                 if (rc && created) {
@@ -1470,7 +1470,8 @@ int mds_mfd_close(struct ptlrpc_request *req, int offset,
                 * rough decision making and will get the proper size later.
                 * This is NOT guaranteed to be correct with multiple
                 * writers, but is only needed until SOM is done. b=11063 */
-               if ((request_body->valid & OBD_MD_FLSIZE) &&
+               if (S_ISREG(inode->i_mode) &&
+                   (request_body->valid & OBD_MD_FLSIZE) &&
                    (iattr.ia_valid != 0)) {
                        iattr.ia_size = request_body->size;
                        iattr.ia_valid |= ATTR_SIZE;
