@@ -1,27 +1,41 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  lustre/mgc/mgc_request.c
- *  Lustre Management Client
+ * GPL HEADER START
  *
- *  Copyright (C) 2006 Cluster File Systems, Inc.
- *   Author: Nathan Rutman <nathan@clusterfs.com>
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   This file is part of Lustre, http://www.lustre.org
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   Lustre is free software; you can redistribute it and/or
- *   modify it under the terms of version 2 of the GNU General Public
- *   License as published by the Free Software Foundation.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   Lustre is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
  *
- *   You should have received a copy of the GNU General Public License
- *   along with Lustre; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
  *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lustre/mgc/mgc_request.c
+ *
+ * Author: Nathan Rutman <nathan@clusterfs.com>
  */
 
 #ifndef EXPORT_SYMTAB
@@ -73,7 +87,7 @@ static int mgc_name2resid(char *name, int len, struct ldlm_res_id *res_id)
 int mgc_fsname2resid(char *fsname, struct ldlm_res_id *res_id)
 {
         /* fsname is at most 8 chars long, maybe contain "-".
-         * e.g. "lustre", "CFS-000" */
+         * e.g. "lustre", "SUN-000" */
         return mgc_name2resid(fsname, strlen(fsname), res_id);
 }
 EXPORT_SYMBOL(mgc_fsname2resid);
@@ -84,7 +98,7 @@ int mgc_logname2resid(char *logname, struct ldlm_res_id *res_id)
         int len;
 
         /* logname consists of "fsname-nodetype".
-         * e.g. "lustre-MDT0001", "CFS-000-client" */
+         * e.g. "lustre-MDT0001", "SUN-000-client" */
         name_end = strrchr(logname, '-');
         LASSERT(name_end);
         len = name_end - logname;
@@ -108,7 +122,7 @@ static int config_log_get(struct config_llog_data *cld)
         RETURN(0);
 }
 
-/* Drop a reference to a config log.  When no longer referenced, 
+/* Drop a reference to a config log.  When no longer referenced,
    we can free the config log data */
 static void config_log_put(struct config_llog_data *cld)
 {
@@ -123,7 +137,7 @@ static void config_log_put(struct config_llog_data *cld)
                 spin_unlock(&config_list_lock);
                 OBD_FREE(cld->cld_logname, strlen(cld->cld_logname) + 1);
                 if (cld->cld_cfg.cfg_instance != NULL)
-                        OBD_FREE(cld->cld_cfg.cfg_instance, 
+                        OBD_FREE(cld->cld_cfg.cfg_instance,
                                  strlen(cld->cld_cfg.cfg_instance) + 1);
                 OBD_FREE(cld, sizeof(*cld));
         }
@@ -131,7 +145,7 @@ static void config_log_put(struct config_llog_data *cld)
 }
 
 /* Find a config log by name */
-static struct config_llog_data *config_log_find(char *logname, 
+static struct config_llog_data *config_log_find(char *logname,
                                                struct config_llog_instance *cfg)
 {
         struct list_head *tmp;
@@ -152,10 +166,10 @@ static struct config_llog_data *config_log_find(char *logname,
         spin_lock(&config_list_lock);
         list_for_each(tmp, &config_llog_list) {
                 cld = list_entry(tmp, struct config_llog_data, cld_list_chain);
-                if (match_instance && cld->cld_cfg.cfg_instance && 
+                if (match_instance && cld->cld_cfg.cfg_instance &&
                     strcmp(logid, cld->cld_cfg.cfg_instance) == 0)
                         goto out_found;
-                if (!match_instance &&  
+                if (!match_instance &&
                     strcmp(logid, cld->cld_logname) == 0)
                         goto out_found;
         }
@@ -169,7 +183,7 @@ out_found:
         RETURN(cld);
 }
 
-/* Add this log to our list of active logs. 
+/* Add this log to our list of active logs.
    We have one active log per "mount" - client instance or servername.
    Each instance may be at a different point in the log. */
 static int config_log_add(char *logname, struct config_llog_instance *cfg,
@@ -181,12 +195,12 @@ static int config_log_add(char *logname, struct config_llog_instance *cfg,
         ENTRY;
 
         CDEBUG(D_MGC, "adding config log %s:%s\n", logname, cfg->cfg_instance);
-        
+
         OBD_ALLOC(cld, sizeof(*cld));
-        if (!cld) 
+        if (!cld)
                 RETURN(-ENOMEM);
         OBD_ALLOC(cld->cld_logname, strlen(logname) + 1);
-        if (!cld->cld_logname) { 
+        if (!cld->cld_logname) {
                 OBD_FREE(cld, sizeof(*cld));
                 RETURN(-ENOMEM);
         }
@@ -196,12 +210,12 @@ static int config_log_add(char *logname, struct config_llog_instance *cfg,
         cld->cld_cfg.cfg_flags = 0;
         cld->cld_cfg.cfg_sb = sb;
         atomic_set(&cld->cld_refcount, 1);
-        
+
         /* Keep the mgc around until we are done */
         cld->cld_mgcexp = class_export_get(lsi->lsi_mgc->obd_self_export);
-        
+
         if (cfg->cfg_instance != NULL) {
-                OBD_ALLOC(cld->cld_cfg.cfg_instance, 
+                OBD_ALLOC(cld->cld_cfg.cfg_instance,
                           strlen(cfg->cfg_instance) + 1);
                 strcpy(cld->cld_cfg.cfg_instance, cfg->cfg_instance);
         }
@@ -209,29 +223,34 @@ static int config_log_add(char *logname, struct config_llog_instance *cfg,
         spin_lock(&config_list_lock);
         list_add(&cld->cld_list_chain, &config_llog_list);
         spin_unlock(&config_list_lock);
-        
+
         if (rc) {
                 config_log_put(cld);
                 RETURN(rc);
         }
-        
+
         RETURN(rc);
 }
 
+DECLARE_MUTEX(llog_process_lock);
+
 /* Stop watching for updates on this log. */
 static int config_log_end(char *logname, struct config_llog_instance *cfg)
-{       
+{
         struct config_llog_data *cld;
         int rc = 0;
         ENTRY;
-                                       
+
         cld = config_log_find(logname, cfg);
-        if (IS_ERR(cld)) 
+        if (IS_ERR(cld))
                 RETURN(PTR_ERR(cld));
         /* drop the ref from the find */
         config_log_put(cld);
 
+        down(&llog_process_lock);
         cld->cld_stopping = 1;
+        up(&llog_process_lock);
+
         /* drop the start ref */
         config_log_put(cld);
         CDEBUG(D_MGC, "end config log %s (%d)\n", logname ? logname : "client",
@@ -247,7 +266,7 @@ static int config_log_end(char *logname, struct config_llog_instance *cfg)
 static int rq_state = 0;
 static cfs_waitq_t rq_waitq;
 
-static int mgc_process_log(struct obd_device *mgc, 
+static int mgc_process_log(struct obd_device *mgc,
                            struct config_llog_data *cld);
 
 static int mgc_requeue_thread(void *data)
@@ -257,9 +276,9 @@ static int mgc_requeue_thread(void *data)
         char name[] = "ll_cfg_requeue";
         int rc = 0;
         ENTRY;
-        
+
         ptlrpc_daemonize(name);
-        
+
         CDEBUG(D_MGC, "Starting requeue thread\n");
 
         lwi_later = LWI_TIMEOUT(60 * HZ, NULL, NULL);
@@ -269,37 +288,37 @@ static int mgc_requeue_thread(void *data)
         spin_lock(&config_list_lock);
         while (rq_state & (RQ_NOW | RQ_LATER)) {
                 /* Any new or requeued lostlocks will change the state */
-                rq_state &= ~(RQ_NOW | RQ_LATER); 
+                rq_state &= ~(RQ_NOW | RQ_LATER);
                 spin_unlock(&config_list_lock);
 
-                /* Always wait a few seconds to allow the server who 
+                /* Always wait a few seconds to allow the server who
                    caused the lock revocation to finish its setup, plus some
                    random so everyone doesn't try to reconnect at once. */
                 lwi_now = LWI_TIMEOUT(3 * HZ + (ll_rand() & 0xff) * (HZ / 100),
                                       NULL, NULL);
                 l_wait_event(rq_waitq, rq_state & RQ_STOP, &lwi_now);
-                
+
                 spin_lock(&config_list_lock);
                 list_for_each_entry_safe(cld, n, &config_llog_list,
                                          cld_list_chain) {
                         spin_unlock(&config_list_lock);
-                        
+
                         if (cld->cld_lostlock) {
-                                CDEBUG(D_MGC, "updating log %s\n", 
+                                CDEBUG(D_MGC, "updating log %s\n",
                                        cld->cld_logname);
                                 cld->cld_lostlock = 0;
                                 rc = mgc_process_log(cld->cld_mgcexp->exp_obd,
                                                      cld);
-                                /* Whether we enqueued again or not in 
-                                   mgc_process_log, we're done with the ref 
-                                   from the old enqueue */        
+                                /* Whether we enqueued again or not in
+                                   mgc_process_log, we're done with the ref
+                                   from the old enqueue */
                                 config_log_put(cld);
                         }
 
                         spin_lock(&config_list_lock);
                 }
                 spin_unlock(&config_list_lock);
-                
+
                 /* Wait a bit to see if anyone else needs a requeue */
                 l_wait_event(rq_waitq, rq_state & (RQ_NOW | RQ_STOP),
                              &lwi_later);
@@ -308,7 +327,7 @@ static int mgc_requeue_thread(void *data)
         /* spinlock and while guarantee RQ_NOW and RQ_LATER are not set */
         rq_state &= ~RQ_RUNNING;
         spin_unlock(&config_list_lock);
-        
+
         CDEBUG(D_MGC, "Ending requeue thread\n");
         RETURN(rc);
 }
@@ -319,13 +338,12 @@ static int mgc_requeue_add(struct config_llog_data *cld, int later)
 {
         int rc = 0;
 
-        CDEBUG(D_INFO, "log %s: requeue (l=%d r=%d sp=%d st=%x)\n", 
+        CDEBUG(D_INFO, "log %s: requeue (l=%d r=%d sp=%d st=%x)\n",
                cld->cld_logname, later, atomic_read(&cld->cld_refcount),
                cld->cld_stopping, rq_state);
-        
+
         /* Hold lock for rq_state */
         spin_lock(&config_list_lock);
-        cld->cld_lostlock = 1;
 
         if (cld->cld_stopping || (rq_state & RQ_STOP)) {
                 spin_unlock(&config_list_lock);
@@ -333,6 +351,8 @@ static int mgc_requeue_add(struct config_llog_data *cld, int later)
                 RETURN(0);
         }
 
+        cld->cld_lostlock = 1;
+
         if (!(rq_state & RQ_RUNNING)) {
                 LASSERT(rq_state == 0);
                 rq_state = RQ_RUNNING | (later ? RQ_LATER : RQ_NOW);
@@ -359,7 +379,7 @@ static int mgc_requeue_add(struct config_llog_data *cld, int later)
 
 /********************** class fns **********************/
 
-static int mgc_fs_setup(struct obd_device *obd, struct super_block *sb, 
+static int mgc_fs_setup(struct obd_device *obd, struct super_block *sb,
                         struct vfsmount *mnt)
 {
         struct lvfs_run_ctxt saved;
@@ -381,7 +401,7 @@ static int mgc_fs_setup(struct obd_device *obd, struct super_block *sb,
         obd->obd_fsops = fsfilt_get_ops(MT_STR(lsi->lsi_ldd));
         if (IS_ERR(obd->obd_fsops)) {
                 up(&cli->cl_mgc_sem);
-                CERROR("No fstype %s rc=%ld\n", MT_STR(lsi->lsi_ldd), 
+                CERROR("No fstype %s rc=%ld\n", MT_STR(lsi->lsi_ldd),
                        PTR_ERR(obd->obd_fsops));
                 RETURN(PTR_ERR(obd->obd_fsops));
         }
@@ -400,7 +420,7 @@ static int mgc_fs_setup(struct obd_device *obd, struct super_block *sb,
         pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
         if (IS_ERR(dentry)) {
                 err = PTR_ERR(dentry);
-                CERROR("cannot lookup %s directory: rc = %d\n", 
+                CERROR("cannot lookup %s directory: rc = %d\n",
                        MOUNT_CONFIGS_DIR, err);
                 GOTO(err_ops, err);
         }
@@ -417,7 +437,7 @@ static int mgc_fs_setup(struct obd_device *obd, struct super_block *sb,
         /* We keep the cl_mgc_sem until mgc_fs_cleanup */
         RETURN(0);
 
-err_ops:        
+err_ops:
         fsfilt_put_ops(obd->obd_fsops);
         obd->obd_fsops = NULL;
         cli->cl_mgc_vfsmnt = NULL;
@@ -437,15 +457,15 @@ static int mgc_fs_cleanup(struct obd_device *obd)
                 struct lvfs_run_ctxt saved;
                 push_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
                 l_dput(cli->cl_mgc_configs_dir);
-                cli->cl_mgc_configs_dir = NULL; 
+                cli->cl_mgc_configs_dir = NULL;
                 pop_ctxt(&saved, &obd->obd_lvfs_ctxt, NULL);
                 class_decref(obd);
         }
 
         cli->cl_mgc_vfsmnt = NULL;
-        if (obd->obd_fsops) 
+        if (obd->obd_fsops)
                 fsfilt_put_ops(obd->obd_fsops);
-        
+
         up(&cli->cl_mgc_sem);
 
         RETURN(rc);
@@ -458,23 +478,23 @@ static int mgc_precleanup(struct obd_device *obd, enum obd_cleanup_stage stage)
         ENTRY;
 
         switch (stage) {
-        case OBD_CLEANUP_EARLY: 
+        case OBD_CLEANUP_EARLY:
                 break;
         case OBD_CLEANUP_EXPORTS:
                 if (atomic_dec_and_test(&mgc_count)) {
-                        /* Kick the requeue waitq - cld's should all be 
+                        /* Kick the requeue waitq - cld's should all be
                            stopping */
                         spin_lock(&config_list_lock);
                         rq_state |= RQ_STOP;
                         spin_unlock(&config_list_lock);
                         cfs_waitq_signal(&rq_waitq);
                 }
-                break;
-        case OBD_CLEANUP_SELF_EXP:
                 rc = obd_llog_finish(obd, 0);
                 if (rc != 0)
                         CERROR("failed to cleanup llogging subsystems\n");
                 break;
+        case OBD_CLEANUP_SELF_EXP:
+                break;
         case OBD_CLEANUP_OBD:
                 break;
         }
@@ -488,10 +508,10 @@ static int mgc_cleanup(struct obd_device *obd)
         ENTRY;
 
         LASSERT(cli->cl_mgc_vfsmnt == NULL);
-        
-        /* COMPAT_146 - old config logs may have added profiles we don't 
+
+        /* COMPAT_146 - old config logs may have added profiles we don't
            know about */
-        if (obd->obd_type->typ_refcnt <= 1) 
+        if (obd->obd_type->typ_refcnt <= 1)
                 /* Only for the last mgc */
                 class_del_profiles();
 
@@ -530,7 +550,7 @@ static int mgc_setup(struct obd_device *obd, obd_count len, void *buf)
                 cfs_waitq_init(&rq_waitq);
         }
         spin_unlock(&config_list_lock);
-        
+
         RETURN(rc);
 
 err_cleanup:
@@ -559,20 +579,20 @@ static int mgc_blocking_ast(struct ldlm_lock *lock, struct ldlm_lock_desc *desc,
         case LDLM_CB_CANCELING: {
                 /* We've given up the lock, prepare ourselves to update. */
                 LDLM_DEBUG(lock, "MGC cancel CB");
-                
+
                 CDEBUG(D_MGC, "Lock res "LPX64" (%.8s)\n",
-                       lock->l_resource->lr_name.name[0], 
+                       lock->l_resource->lr_name.name[0],
                        (char *)&lock->l_resource->lr_name.name[0]);
-                
+
                 if (!cld) {
                         CERROR("missing data, won't requeue\n");
                         break;
                 }
                 /* Are we done with this log? */
                 if (cld->cld_stopping) {
-                        CDEBUG(D_MGC, "log %s: stopping, won't requeue\n", 
+                        CDEBUG(D_MGC, "log %s: stopping, won't requeue\n",
                                cld->cld_logname);
-                        config_log_put(cld);    
+                        config_log_put(cld);
                         break;
                 }
                 /* Make sure not to re-enqueue when the mgc is stopping
@@ -581,7 +601,7 @@ static int mgc_blocking_ast(struct ldlm_lock *lock, struct ldlm_lock_desc *desc,
                     !lock->l_conn_export->exp_obd->u.cli.cl_conn_count) {
                         CDEBUG(D_MGC, "log %s: disconnecting, won't requeue\n",
                                cld->cld_logname);
-                        config_log_put(cld);    
+                        config_log_put(cld);
                         break;
                 }
                 /* Did we fail to get the lock? */
@@ -602,7 +622,7 @@ static int mgc_blocking_ast(struct ldlm_lock *lock, struct ldlm_lock_desc *desc,
 
 
         if (rc) {
-                CERROR("%s CB failed %d:\n", flag == LDLM_CB_BLOCKING ? 
+                CERROR("%s CB failed %d:\n", flag == LDLM_CB_BLOCKING ?
                        "blocking" : "cancel", rc);
                 LDLM_ERROR(lock, "MGC ast");
         }
@@ -615,7 +635,7 @@ static int mgc_enqueue(struct obd_export *exp, struct lov_stripe_md *lsm,
                        int *flags, void *bl_cb, void *cp_cb, void *gl_cb,
                        void *data, __u32 lvb_len, void *lvb_swabber,
                        struct lustre_handle *lockh)
-{                       
+{
         struct config_llog_data *cld = (struct config_llog_data *)data;
         struct ldlm_enqueue_info einfo = { type, mode, mgc_blocking_ast,
                 ldlm_completion_ast, NULL, data};
@@ -625,7 +645,7 @@ static int mgc_enqueue(struct obd_export *exp, struct lov_stripe_md *lsm,
 
         CDEBUG(D_MGC, "Enqueue for %s (res "LPX64")\n", cld->cld_logname,
                cld->cld_resid.name[0]);
-                
+
         /* We can only drop this config log ref when we drop the lock */
         if (config_log_get(cld))
                 RETURN(ELDLM_LOCK_ABORTED);
@@ -635,8 +655,8 @@ static int mgc_enqueue(struct obd_export *exp, struct lov_stripe_md *lsm,
 
         rc = ldlm_cli_enqueue(exp, NULL, &einfo, cld->cld_resid,
                               NULL, flags, NULL, 0, NULL, lockh, 0);
-        /* A failed enqueue should still call the mgc_blocking_ast, 
-           where it will be requeued if needed ("grant failed"). */ 
+        /* A failed enqueue should still call the mgc_blocking_ast,
+           where it will be requeued if needed ("grant failed"). */
 
         RETURN(rc);
 }
@@ -662,14 +682,10 @@ static int mgc_iocontrol(unsigned int cmd, struct obd_export *exp, int len,
         int rc;
         ENTRY;
 
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
-        MOD_INC_USE_COUNT;
-#else
         if (!try_module_get(THIS_MODULE)) {
                 CERROR("Can't get module. Is it alive?");
                 return -EINVAL;
         }
-#endif
         switch (cmd) {
         /* REPLicator context */
         case OBD_IOC_PARSE: {
@@ -703,11 +719,7 @@ static int mgc_iocontrol(unsigned int cmd, struct obd_export *exp, int len,
                 GOTO(out, rc = -ENOTTY);
         }
 out:
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0))
-        MOD_DEC_USE_COUNT;
-#else
         module_put(THIS_MODULE);
-#endif
 
         return rc;
 }
@@ -730,14 +742,15 @@ static int mgc_target_register(struct obd_export *exp,
                 RETURN(-ENOMEM);
 
         req_mti = lustre_msg_buf(req->rq_reqmsg, REQ_REC_OFF, sizeof(*req_mti));
-        if (!req_mti) 
+        if (!req_mti) {
+                ptlrpc_req_finished(req);
                 RETURN(-ENOMEM);
+        }
+
         memcpy(req_mti, mti, sizeof(*req_mti));
-
         ptlrpc_req_set_repsize(req, 2, rep_size);
-
         CDEBUG(D_MGC, "register %s\n", mti->mti_svname);
-        
+
         rc = ptlrpc_queue_wait(req);
         if (!rc) {
                 rep_mti = lustre_swab_repbuf(req, REPLY_REC_OFF,
@@ -759,7 +772,7 @@ static int mgc_set_mgs_param(struct obd_export *exp,
         struct ptlrpc_request *req;
         struct mgs_send_param *req_msp, *rep_msp;
         int size[] = { sizeof(struct ptlrpc_body), sizeof(*req_msp) };
-        int rep_size[] = { sizeof(struct ptlrpc_body), sizeof(*msp) };
+        __u32 rep_size[] = { sizeof(struct ptlrpc_body), sizeof(*msp) };
         int rc;
         ENTRY;
 
@@ -769,11 +782,14 @@ static int mgc_set_mgs_param(struct obd_export *exp,
                 RETURN(-ENOMEM);
 
         req_msp = lustre_msg_buf(req->rq_reqmsg, REQ_REC_OFF, sizeof(*req_msp));
-        if (!req_msp)
+        if (!req_msp) {
+                ptlrpc_req_finished(req);
                 RETURN(-ENOMEM);
+        }
 
         memcpy(req_msp, msp, sizeof(*req_msp));
         ptlrpc_req_set_repsize(req, 2, rep_size);
+
         rc = ptlrpc_queue_wait(req);
         if (!rc) {
                 rep_msp = lustre_swab_repbuf(req, REPLY_REC_OFF,
@@ -787,7 +803,7 @@ static int mgc_set_mgs_param(struct obd_export *exp,
 }
 
 int mgc_set_info_async(struct obd_export *exp, obd_count keylen,
-                       void *key, obd_count vallen, void *val, 
+                       void *key, obd_count vallen, void *val,
                        struct ptlrpc_request_set *set)
 {
         struct obd_import *imp = class_exp2cliimp(exp);
@@ -813,22 +829,22 @@ int mgc_set_info_async(struct obd_export *exp, obd_count keylen,
                 value = *(int *)val;
                 spin_lock(&imp->imp_lock);
                 imp->imp_initial_recov_bk = value > 0;
-                /* Even after the initial connection, give up all comms if 
+                /* Even after the initial connection, give up all comms if
                    nobody answers the first time. */
                 imp->imp_recon_bk = 1;
                 spin_unlock(&imp->imp_lock);
-                CDEBUG(D_MGC, "InitRecov %s %d/%d:d%d:i%d:r%d:or%d:%s\n", 
+                CDEBUG(D_MGC, "InitRecov %s %d/%d:d%d:i%d:r%d:or%d:%s\n",
                        imp->imp_obd->obd_name, value, imp->imp_initial_recov,
-                       imp->imp_deactive, imp->imp_invalid, 
+                       imp->imp_deactive, imp->imp_invalid,
                        imp->imp_replayable, imp->imp_obd->obd_replayable,
                        ptlrpc_import_state_name(imp->imp_state));
                 /* Resurrect if we previously died */
-                if (imp->imp_invalid || value > 1) 
+                if (imp->imp_invalid || value > 1)
                         ptlrpc_reconnect_import(imp);
                 RETURN(0);
         }
         /* FIXME move this to mgc_process_config */
-        if (KEY_IS("register_target")) {
+        if (KEY_IS(KEY_REGISTER_TARGET)) {
                 struct mgs_target_info *mti;
                 if (vallen != sizeof(struct mgs_target_info))
                         RETURN(-EINVAL);
@@ -838,7 +854,7 @@ int mgc_set_info_async(struct obd_export *exp, obd_count keylen,
                 rc =  mgc_target_register(exp, mti);
                 RETURN(rc);
         }
-        if (KEY_IS("set_fs")) {
+        if (KEY_IS(KEY_SET_FS)) {
                 struct super_block *sb = (struct super_block *)val;
                 struct lustre_sb_info *lsi;
                 if (vallen != sizeof(struct super_block))
@@ -850,7 +866,7 @@ int mgc_set_info_async(struct obd_export *exp, obd_count keylen,
                 }
                 RETURN(rc);
         }
-        if (KEY_IS("clear_fs")) {
+        if (KEY_IS(KEY_CLEAR_FS)) {
                 if (vallen != 0)
                         RETURN(-EINVAL);
                 rc = mgc_fs_cleanup(exp->exp_obd);
@@ -880,17 +896,17 @@ static int mgc_import_event(struct obd_device *obd,
         CDEBUG(D_MGC, "import event %#x\n", event);
 
         switch (event) {
-        case IMP_EVENT_DISCON: 
+        case IMP_EVENT_DISCON:
                 /* MGC imports should not wait for recovery */
                 break;
-        case IMP_EVENT_INACTIVE: 
+        case IMP_EVENT_INACTIVE:
                 break;
         case IMP_EVENT_INVALIDATE: {
                 struct ldlm_namespace *ns = obd->obd_namespace;
                 ldlm_namespace_cleanup(ns, LDLM_FL_LOCAL_ONLY);
                 break;
         }
-        case IMP_EVENT_ACTIVE: 
+        case IMP_EVENT_ACTIVE:
                 LCONSOLE_WARN("%s: Reactivating import\n", obd->obd_name);
                 /* Clearing obd_no_recov allows us to continue pinging */
                 obd->obd_no_recov = 0;
@@ -921,8 +937,12 @@ static int mgc_llog_init(struct obd_device *obd, struct obd_device *tgt,
                         &llog_client_ops);
         if (rc == 0) {
                 ctxt = llog_get_context(obd, LLOG_CONFIG_REPL_CTXT);
-                ctxt->loc_imp = obd->u.cli.cl_import;
+                llog_initiator_connect(ctxt);
                 llog_ctxt_put(ctxt);
+        } else {
+                ctxt = llog_get_context(obd, LLOG_CONFIG_ORIG_CTXT);
+                if (ctxt)
+                        llog_cleanup(ctxt);
         }
 
         RETURN(rc);
@@ -959,7 +979,7 @@ static int mgc_llog_is_empty(struct obd_device *obd, struct llog_ctxt *ctxt,
         return(rc <= 1);
 }
 
-static int mgc_copy_handler(struct llog_handle *llh, struct llog_rec_hdr *rec, 
+static int mgc_copy_handler(struct llog_handle *llh, struct llog_rec_hdr *rec,
                             void *data)
 {
         struct llog_rec_hdr local_rec = *rec;
@@ -971,12 +991,12 @@ static int mgc_copy_handler(struct llog_handle *llh, struct llog_rec_hdr *rec,
 
         /* Append all records */
         local_rec.lrh_len -= sizeof(*rec) + sizeof(struct llog_rec_tail);
-        rc = llog_write_rec(local_llh, &local_rec, NULL, 0, 
+        rc = llog_write_rec(local_llh, &local_rec, NULL, 0,
                             (void *)cfg_buf, -1);
 
         lcfg = (struct lustre_cfg *)cfg_buf;
-        CDEBUG(D_INFO, "idx=%d, rc=%d, len=%d, cmd %x %s %s\n", 
-               rec->lrh_index, rc, rec->lrh_len, lcfg->lcfg_command, 
+        CDEBUG(D_INFO, "idx=%d, rc=%d, len=%d, cmd %x %s %s\n",
+               rec->lrh_index, rc, rec->lrh_len, lcfg->lcfg_command,
                lustre_cfg_string(lcfg, 0), lustre_cfg_string(lcfg, 1));
 
         RETURN(rc);
@@ -996,7 +1016,7 @@ static int mgc_copy_llog(struct obd_device *obd, struct llog_ctxt *rctxt,
            upon successful completion. */
 
         OBD_ALLOC(temp_log, strlen(logname) + 1);
-        if (!temp_log) 
+        if (!temp_log)
                 RETURN(-ENOMEM);
         sprintf(temp_log, "%sT", logname);
 
@@ -1005,7 +1025,7 @@ static int mgc_copy_llog(struct obd_device *obd, struct llog_ctxt *rctxt,
         if (rc)
                 GOTO(out, rc);
         rc = llog_init_handle(local_llh, LLOG_F_IS_PLAIN, NULL);
-        if (rc) 
+        if (rc)
                 GOTO(out, rc);
         rc = llog_destroy(local_llh);
         llog_free_handle(local_llh);
@@ -1051,7 +1071,8 @@ out_closel:
                 struct client_obd *cli = &obd->u.cli;
                 LASSERT(cli);
                 LASSERT(cli->cl_mgc_configs_dir);
-                rc = lustre_rename(cli->cl_mgc_configs_dir, temp_log, logname);
+                rc = lustre_rename(cli->cl_mgc_configs_dir, cli->cl_mgc_vfsmnt,
+                                   temp_log, logname);
         }
         CDEBUG(D_MGC, "Copied remote log %s (%d)\n", logname, rc);
 out:
@@ -1061,11 +1082,9 @@ out:
         RETURN(rc);
 }
 
-DECLARE_MUTEX(llog_process_lock);
-
 /* Get a config log from the MGS and process it.
    This func is called for both clients and servers. */
-static int mgc_process_log(struct obd_device *mgc, 
+static int mgc_process_log(struct obd_device *mgc,
                            struct config_llog_data *cld)
 {
         struct llog_ctxt *ctxt, *lctxt;
@@ -1081,45 +1100,49 @@ static int mgc_process_log(struct obd_device *mgc,
                 CERROR("Missing cld, aborting log update\n");
                 RETURN(-EINVAL);
         }
-        if (cld->cld_stopping) 
+
+        /* I don't want mutliple processes running process_log at once --
+           sounds like badness.  It actually might be fine, as long as
+           we're not trying to update from the same log
+           simultaneously (in which case we should use a per-log sem.) */
+        down(&llog_process_lock);
+
+        if (cld->cld_stopping) {
+                up(&llog_process_lock);
                 RETURN(0);
+        }
 
         OBD_FAIL_TIMEOUT(OBD_FAIL_MGC_PAUSE_PROCESS_LOG, 20);
 
         lsi = s2lsi(cld->cld_cfg.cfg_sb);
 
-        CDEBUG(D_MGC, "Process log %s:%s from %d\n", cld->cld_logname, 
+        CDEBUG(D_MGC, "Process log %s:%s from %d\n", cld->cld_logname,
                cld->cld_cfg.cfg_instance, cld->cld_cfg.cfg_last_idx + 1);
 
         ctxt = llog_get_context(mgc, LLOG_CONFIG_REPL_CTXT);
         if (!ctxt) {
                 CERROR("missing llog context\n");
+                up(&llog_process_lock);
                 RETURN(-EINVAL);
         }
 
-        /* I don't want mutliple processes running process_log at once -- 
-           sounds like badness.  It actually might be fine, as long as 
-           we're not trying to update from the same log
-           simultaneously (in which case we should use a per-log sem.) */
-        down(&llog_process_lock);
-
         /* Get the cfg lock on the llog */
-        rcl = mgc_enqueue(mgc->u.cli.cl_mgc_mgsexp, NULL, LDLM_PLAIN, NULL, 
-                          LCK_CR, &flags, NULL, NULL, NULL, 
+        rcl = mgc_enqueue(mgc->u.cli.cl_mgc_mgsexp, NULL, LDLM_PLAIN, NULL,
+                          LCK_CR, &flags, NULL, NULL, NULL,
                           cld, 0, NULL, &lockh);
-        if (rcl) 
+        if (rcl)
                 CDEBUG(D_MGC, "Can't get cfg lock: %d\n", rcl);
-        
+
         lctxt = llog_get_context(mgc, LLOG_CONFIG_ORIG_CTXT);
 
-        /* Copy the setup log locally if we can. Don't mess around if we're 
+        /* Copy the setup log locally if we can. Don't mess around if we're
            running an MGS though (logs are already local). */
-        if (lctxt && lsi && (lsi->lsi_flags & LSI_SERVER) && 
+        if (lctxt && lsi && (lsi->lsi_flags & LSI_SERVER) &&
             (lsi->lsi_srv_mnt == cli->cl_mgc_vfsmnt) &&
             !IS_MGS(lsi->lsi_ldd)) {
                 push_ctxt(&saved, &mgc->obd_lvfs_ctxt, NULL);
                 must_pop++;
-                if (rcl == 0) 
+                if (rcl == 0)
                         /* Only try to copy log if we have the lock. */
                         rc = mgc_copy_llog(mgc, ctxt, lctxt, cld->cld_logname);
                 if (rcl || rc) {
@@ -1134,37 +1157,37 @@ static int mgc_process_log(struct obd_device *mgc,
                                cld->cld_logname);
                 }
                 /* Now, whether we copied or not, start using the local llog.
-                   If we failed to copy, we'll start using whatever the old 
+                   If we failed to copy, we'll start using whatever the old
                    log has. */
                 llog_ctxt_put(ctxt);
                 ctxt = lctxt;
         }
 
-        /* logname and instance info should be the same, so use our 
+        /* logname and instance info should be the same, so use our
            copy of the instance for the update.  The cfg_last_idx will
            be updated here. */
         rc = class_config_parse_llog(ctxt, cld->cld_logname, &cld->cld_cfg);
- 
+
 out_pop:
         llog_ctxt_put(ctxt);
         if (ctxt != lctxt)
                 llog_ctxt_put(lctxt);
-        if (must_pop) 
+        if (must_pop)
                 pop_ctxt(&saved, &mgc->obd_lvfs_ctxt, NULL);
 
-        /* Now drop the lock so MGS can revoke it */ 
+        /* Now drop the lock so MGS can revoke it */
         if (!rcl) {
-                rcl = mgc_cancel(mgc->u.cli.cl_mgc_mgsexp, NULL, 
+                rcl = mgc_cancel(mgc->u.cli.cl_mgc_mgsexp, NULL,
                                  LCK_CR, &lockh);
-                if (rcl) 
+                if (rcl)
                         CERROR("Can't drop cfg lock: %d\n", rcl);
         }
-        
+
         CDEBUG(D_MGC, "%s: configuration from log '%s' %sed (%d).\n",
                mgc->obd_name, cld->cld_logname, rc ? "fail" : "succeed", rc);
 
         up(&llog_process_lock);
-        
+
         RETURN(rc);
 }
 
@@ -1180,18 +1203,18 @@ static int mgc_process_config(struct obd_device *obd, obd_count len, void *buf)
                 /* Add any new target, not just osts */
                 struct mgs_target_info *mti;
 
-                if (LUSTRE_CFG_BUFLEN(lcfg, 1) != 
+                if (LUSTRE_CFG_BUFLEN(lcfg, 1) !=
                     sizeof(struct mgs_target_info))
                         GOTO(out, rc = -EINVAL);
 
                 mti = (struct mgs_target_info *)lustre_cfg_buf(lcfg, 1);
-                CDEBUG(D_MGC, "add_target %s %#x\n",    
+                CDEBUG(D_MGC, "add_target %s %#x\n",
                        mti->mti_svname, mti->mti_flags);
                 rc = mgc_target_register(obd->u.cli.cl_mgc_mgsexp, mti);
                 break;
         }
         case LCFG_LOV_DEL_OBD:
-                /* Remove target from the fs? */  
+                /* Remove target from the fs? */
                 /* FIXME */
                 CERROR("lov_del_obd unimplemented\n");
                 rc = -ENOSYS;
@@ -1203,29 +1226,29 @@ static int mgc_process_config(struct obd_device *obd, obd_count len, void *buf)
                 char *logname = lustre_cfg_string(lcfg, 1);
                 cfg = (struct config_llog_instance *)lustre_cfg_buf(lcfg, 2);
                 sb = *(struct super_block **)lustre_cfg_buf(lcfg, 3);
-                
-                CDEBUG(D_MGC, "parse_log %s from %d\n", logname, 
+
+                CDEBUG(D_MGC, "parse_log %s from %d\n", logname,
                        cfg->cfg_last_idx);
 
                 /* We're only called through here on the initial mount */
                 rc = config_log_add(logname, cfg, sb);
-                if (rc) 
+                if (rc)
                         break;
                 cld = config_log_find(logname, cfg);
                 if (IS_ERR(cld)) {
                         rc = PTR_ERR(cld);
                         break;
                 }
-                
+
                 /* COMPAT_146 */
                 /* FIXME only set this for old logs!  Right now this forces
                    us to always skip the "inside markers" check */
                 cld->cld_cfg.cfg_flags |= CFG_F_COMPAT146;
-                
+
                 rc = mgc_process_log(obd, cld);
                 config_log_put(cld);
-                
-                break;       
+
+                break;
         }
         case LCFG_LOG_END: {
                 struct config_llog_instance *cfg = NULL;
@@ -1276,7 +1299,7 @@ static void /*__exit*/ mgc_exit(void)
         class_unregister_type(LUSTRE_MGC_NAME);
 }
 
-MODULE_AUTHOR("Cluster File Systems, Inc. <info@clusterfs.com>");
+MODULE_AUTHOR("Sun Microsystems, Inc. <http://www.lustre.org/>");
 MODULE_DESCRIPTION("Lustre Management Client");
 MODULE_LICENSE("GPL");
 
