@@ -1,33 +1,42 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  Copyright (C) 2002, 2003 Cluster File Systems, Inc.
+ * GPL HEADER START
  *
- *   This file is part of the Lustre file system, http://www.lustre.org
- *   Lustre is a trademark of Cluster File Systems, Inc.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   You may have signed or agreed to another license before downloading
- *   this software.  If so, you are bound by the terms and conditions
- *   of that agreement, and the following does not apply to you.  See the
- *   LICENSE file included with this distribution for more information.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   If you did not agree to a different license, then this copy of Lustre
- *   is open source software; you can redistribute it and/or modify it
- *   under the terms of version 2 of the GNU General Public License as
- *   published by the Free Software Foundation.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   In either case, Lustre is distributed in the hope that it will be
- *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty
- *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   license text for more details.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
  *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
  */
 #define DEBUG_SUBSYSTEM S_CLASS
 
 #include <linux/version.h>
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2,5,0))
 #include <asm/statfs.h>
-#endif
 #include <obd_class.h>
 #include <lprocfs_status.h>
 #include <linux/seq_file.h>
@@ -51,7 +60,7 @@ static int osc_wr_active(struct file *file, const char *buffer,
 {
         struct obd_device *dev = data;
         int val, rc;
-        
+
         rc = lprocfs_write_helper(buffer, count, &val);
         if (rc)
                 return rc;
@@ -60,11 +69,11 @@ static int osc_wr_active(struct file *file, const char *buffer,
 
         LPROCFS_CLIMP_CHECK(dev);
         /* opposite senses */
-        if (dev->u.cli.cl_import->imp_deactive == val) 
+        if (dev->u.cli.cl_import->imp_deactive == val)
                 rc = ptlrpc_set_import_active(dev->u.cli.cl_import, val);
         else
                 CDEBUG(D_CONFIG, "activate %d: ignoring repeat request\n", val);
-        
+
         LPROCFS_CLIMP_EXIT(dev);
         return count;
 }
@@ -228,6 +237,53 @@ static int osc_rd_create_count(char *page, char **start, off_t off, int count,
 
 static int osc_wr_create_count(struct file *file, const char *buffer,
                                unsigned long count, void *data)
+{
+        struct obd_device *obd = data;
+        int val, rc, i;
+
+        if (obd == NULL)
+                return 0;
+
+        rc = lprocfs_write_helper(buffer, count, &val);
+        if (rc)
+                return rc;
+
+        /* The MDT ALWAYS needs to limit the precreate count to
+         * OST_MAX_PRECREATE, and the constant cannot be changed
+         * because it is a value shared between the OSC and OST
+         * that is the maximum possible number of objects that will
+         * ever be handled by MDT->OST recovery processing.
+         *
+         * If the OST ever gets a request to delete more orphans,
+         * this implies that something has gone badly on the MDT
+         * and the OST will refuse to delete so much data from the
+         * filesystem as a safety measure. */
+        if (val < OST_MIN_PRECREATE || val > OST_MAX_PRECREATE)
+                return -ERANGE;
+        if (val > obd->u.cli.cl_oscc.oscc_max_grow_count)
+                return -ERANGE;
+
+        for (i = 1; (i << 1) <= val; i <<= 1)
+                ;
+        obd->u.cli.cl_oscc.oscc_grow_count = i;
+
+        return count;
+}
+
+static int osc_rd_max_create_count(char *page, char **start, off_t off,
+                                   int count, int *eof, void *data)
+{
+        struct obd_device *obd = data;
+
+        if (obd == NULL)
+                return 0;
+
+        return snprintf(page, count, "%d\n",
+                        obd->u.cli.cl_oscc.oscc_max_grow_count);
+}
+
+static int osc_wr_max_create_count(struct file *file, const char *buffer,
+                                   unsigned long count, void *data)
 {
         struct obd_device *obd = data;
         int val, rc;
@@ -244,7 +300,10 @@ static int osc_wr_create_count(struct file *file, const char *buffer,
         if (val > OST_MAX_PRECREATE)
                 return -ERANGE;
 
-        obd->u.cli.cl_oscc.oscc_grow_count = val;
+        if (obd->u.cli.cl_oscc.oscc_grow_count > val)
+                obd->u.cli.cl_oscc.oscc_grow_count = val;
+
+        obd->u.cli.cl_oscc.oscc_max_grow_count = val;
 
         return count;
 }
@@ -364,7 +423,7 @@ static int osc_rd_resend_count(char *page, char **start, off_t off, int count,
 {
         struct obd_device *obd = data;
 
-        return snprintf(page, count, "%u\n", atomic_read(&obd->u.cli.cl_resends)); 
+        return snprintf(page, count, "%u\n", atomic_read(&obd->u.cli.cl_resends));
 }
 
 static int osc_wr_resend_count(struct file *file, const char *buffer,
@@ -387,7 +446,7 @@ static int osc_wr_resend_count(struct file *file, const char *buffer,
 
 static struct lprocfs_vars lprocfs_osc_obd_vars[] = {
         { "uuid",            lprocfs_rd_uuid,        0, 0 },
-        { "ping",            0, lprocfs_wr_ping,        0 },
+        { "ping",            0, lprocfs_wr_ping,     0, 0, 0222 },
         { "connect_flags",   lprocfs_rd_connect_flags, 0, 0 },
         { "blocksize",       lprocfs_rd_blksize,     0, 0 },
         { "kbytestotal",     lprocfs_rd_kbytestotal, 0, 0 },
@@ -398,7 +457,7 @@ static struct lprocfs_vars lprocfs_osc_obd_vars[] = {
         //{ "filegroups",      lprocfs_rd_filegroups,  0, 0 },
         { "ost_server_uuid", lprocfs_rd_server_uuid, 0, 0 },
         { "ost_conn_uuid",   lprocfs_rd_conn_uuid, 0, 0 },
-        { "active",          osc_rd_active, 
+        { "active",          osc_rd_active,
                              osc_wr_active, 0 },
         { "max_pages_per_rpc", osc_rd_max_pages_per_rpc,
                                osc_wr_max_pages_per_rpc, 0 },
@@ -408,12 +467,15 @@ static struct lprocfs_vars lprocfs_osc_obd_vars[] = {
         { "cur_dirty_bytes", osc_rd_cur_dirty_bytes, 0, 0 },
         { "cur_grant_bytes", osc_rd_cur_grant_bytes, 0, 0 },
         { "create_count",    osc_rd_create_count, osc_wr_create_count, 0 },
+        { "max_create_count", osc_rd_max_create_count,
+                              osc_wr_max_create_count, 0},
         { "prealloc_next_id", osc_rd_prealloc_next_id, 0, 0 },
         { "prealloc_last_id", osc_rd_prealloc_last_id, 0, 0 },
         { "checksums",       osc_rd_checksum, osc_wr_checksum, 0 },
         { "checksum_type",   osc_rd_checksum_type, osc_wd_checksum_type, 0 },
         { "resend_count",  osc_rd_resend_count, osc_wr_resend_count, 0},
         { "timeouts",        lprocfs_rd_timeouts,      0, 0 },
+        { "import",          lprocfs_rd_import,    0, 0 },
         { 0 }
 };
 
@@ -509,7 +571,7 @@ static int osc_rpc_stats_seq_show(struct seq_file *seq, void *v)
                 write_cum += w;
                 seq_printf(seq, "%d:\t\t%10lu %3lu %3lu   | %10lu %3lu %3lu\n",
                            (i == 0) ? 0 : 1 << (i - 1),
-                           r, pct(r, read_tot), pct(read_cum, read_tot), 
+                           r, pct(r, read_tot), pct(read_cum, read_tot),
                            w, pct(w, write_tot), pct(write_cum, write_tot));
                 if (read_cum == read_tot && write_cum == write_tot)
                         break;
@@ -552,4 +614,3 @@ void lprocfs_osc_init_vars(struct lprocfs_static_vars *lvars)
         lvars->obd_vars    = lprocfs_osc_obd_vars;
 }
 #endif /* LPROCFS */
-
