@@ -7,22 +7,37 @@ ALWAYS_EXCEPT="15c   $REPLAY_DUAL_EXCEPT"
 
 SAVE_PWD=$PWD
 PTLDEBUG=${PTLDEBUG:--1}
-LUSTRE=${LUSTRE:-`dirname $0`/..}
+LUSTRE=${LUSTRE:-$(cd $(dirname $0)/..; echo $PWD)}
 SETUP=${SETUP:-""}
 CLEANUP=${CLEANUP:-""}
 MOUNT_2=${MOUNT_2:-"yes"}
 . $LUSTRE/tests/test-framework.sh
 
+if [ "$FAILURE_MODE" = "HARD" ] && mixed_ost_devs; then
+    CONFIG_EXCEPTIONS="17"
+    echo -n "Several ost services on one ost node are used with FAILURE_MODE=$FAILURE_MODE. "
+    echo "Except the tests: $CONFIG_EXCEPTIONS"
+    ALWAYS_EXCEPT="$ALWAYS_EXCEPT $CONFIG_EXCEPTIONS"
+fi
+
 init_test_env $@
 
 . ${CONFIG:=$LUSTRE/tests/cfg/$NAME.sh}
 
-#
+remote_mds_nodsh && skip "remote MDS with nodsh" && exit 0
+
 [ "$SLOW" = "no" ] && EXCEPT_SLOW="1 2 3 4 5 14"
 
 build_test_filter
 
-cleanup_and_setup_lustre
+check_and_setup_lustre
+MOUNTED=$(mounted_lustre_filesystems)
+if ! $(echo $MOUNTED | grep -w -q $MOUNT2); then
+    zconf_mount $HOSTNAME $MOUNT2
+    MOUNTED2=yes
+fi
+
+assert_DIR
 rm -rf $DIR/[df][0-9]*
 
 [ "$DAEMONFILE" ] && $LCTL debug_daemon start $DAEMONFILE $DAEMONSIZE
@@ -141,9 +156,9 @@ test_9() {
     mcreate $MOUNT1/$tfile-1
     mcreate $MOUNT2/$tfile-2
     # drop first reint reply
-    do_facet mds sysctl -w lustre.fail_loc=0x80000119
+    do_facet mds lctl set_param fail_loc=0x80000119
     fail mds
-    do_facet mds sysctl -w lustre.fail_loc=0
+    do_facet mds lctl set_param fail_loc=0
 
     rm $MOUNT1/$tfile-[1,2] || return 1
 
@@ -157,9 +172,9 @@ test_10() {
     munlink $MOUNT1/$tfile-1
     mcreate $MOUNT2/$tfile-2
     # drop first reint reply
-    do_facet mds sysctl -w lustre.fail_loc=0x80000119
+    do_facet mds lctl set_param fail_loc=0x80000119
     fail mds
-    do_facet mds sysctl -w lustre.fail_loc=0
+    do_facet mds lctl set_param fail_loc=0
 
     checkstat $MOUNT1/$tfile-1 && return 1
     checkstat $MOUNT1/$tfile-2 || return 2
@@ -177,12 +192,12 @@ test_11() {
     mcreate $MOUNT2/$tfile-4
     mcreate $MOUNT1/$tfile-5
     # drop all reint replies for a while
-    do_facet mds sysctl -w lustre.fail_loc=0x0119
+    do_facet mds lctl set_param fail_loc=0x0119
     # note that with this fail_loc set, facet_failover df will fail
     facet_failover mds
     #sleep for while, let both clients reconnect and timeout
     sleep $((TIMEOUT * 2))
-    do_facet mds sysctl -w lustre.fail_loc=0
+    do_facet mds lctl set_param fail_loc=0
     client_df
     while [ -z "$(ls $MOUNT1/$tfile-[1-5] 2>/dev/null)" ]; do
 	sleep 5
@@ -202,9 +217,9 @@ test_12() {
     MULTIPID=$!
 
 #define OBD_FAIL_LDLM_ENQUEUE            0x302
-    do_facet mds sysctl -w lustre.fail_loc=0x80000302
+    do_facet mds lctl set_param fail_loc=0x80000302
     facet_failover mds
-    do_facet mds sysctl -w lustre.fail_loc=0
+    do_facet mds lctl set_param fail_loc=0
     df $MOUNT || { kill -USR1 $MULTIPID  && return 1; }
 
     ls $DIR/$tfile
@@ -227,9 +242,9 @@ test_13() {
     wait $MULTIPID || return 4
 
     # drop close 
-    do_facet mds sysctl -w lustre.fail_loc=0x80000115
+    do_facet mds lctl set_param fail_loc=0x80000115
     facet_failover mds
-    do_facet mds sysctl -w lustre.fail_loc=0
+    do_facet mds lctl set_param fail_loc=0
     df $MOUNT || return 1
 
     ls $DIR/$tfile
@@ -260,7 +275,7 @@ test_14() {
 }
 run_test 14 "timeouts waiting for lost client during replay"
 
-test_15() {
+test_15a() {	# was test_15
     replay_barrier mds
     createmany -o $MOUNT1/$tfile- 25
     createmany -o $MOUNT2/$tfile-2- 1
@@ -275,72 +290,7 @@ test_15() {
     zconf_mount `hostname` $MOUNT2 || error "mount $MOUNT2 fail"
     return 0
 }
-run_test 15 "timeout waiting for lost client during replay, 1 client completes"
-
-test_15a() {
-    local ost_last_id=""
-    local osc_last_id=""
-    
-    replay_barrier mds
-    echo "data" > "$MOUNT2/${tfile}-m2"
-
-    umount $MOUNT2
-    facet_failover mds
-    df $MOUNT || return 1
-    
-    ost_last_id=`cat /proc/fs/lustre/obdfilter/*/last_id`
-    mds_last_id=`cat /proc/fs/lustre/osc/*mds*/last_id`
-    
-    echo "Ids after MDS<->OST synchonizing"
-    echo "--------------------------------"
-    echo "MDS last_id:"
-    echo $mds_last_id
-    echo "OST last_id:"
-    echo $ost_last_id
-
-    local i=0
-    echo $ost_last_id | while read id; do
-	ost_ids[$i]=$id
-	((i++))
-    done
-    
-    i=0
-    echo $mds_last_id | while read id; do
-	mds_ids[$i]=$id
-	((i++))
-    done
-    
-    local arr_len=${#mds_ids[*]}
-    for ((i=0;i<$arr_len;i++)); do
-	    mds_id=${mds_ids[i]}
-	    ost_id=${ost_ids[i]}
-	    
-	    test $mds_id -ge $ost_id || {
-		echo "MDS last id ($mds_id) is smaller than OST one ($ost_id)"
-		return 2
-	    }
-    done
-
-    zconf_mount `hostname` $MOUNT2 || error "mount $MOUNT2 fail"
-    return 0
-}
-#CROW run_test 15a "OST clear orphans - synchronize ids on MDS and OST"
-
-test_15b() {
-    replay_barrier mds
-    echo "data" > "$MOUNT2/${tfile}-m2"
-    umount $MOUNT2
-
-    do_facet ost1 "sysctl -w lustre.fail_loc=0x80000802"
-    facet_failover mds
-
-    df $MOUNT || return 1
-    do_facet ost1 "sysctl -w lustre.fail_loc=0"
-    
-    zconf_mount `hostname` $MOUNT2 || error "mount $MOUNT2 fail"
-    return 0
-}
-#CROW run_test 15b "multiple delayed OST clear orphans"
+run_test 15a "timeout waiting for lost client during replay, 1 client completes"
 
 test_15c() {
     replay_barrier mds
@@ -378,6 +328,8 @@ test_16() {
 run_test 16 "fail MDS during recovery (3571)"
 
 test_17() {
+    remote_ost_nodsh && skip "remote OST with nodsh" && return 0
+
     createmany -o $MOUNT1/$tfile- 25
     createmany -o $MOUNT2/$tfile-2- 1
 
@@ -409,10 +361,10 @@ test_18() { # bug 3822 - evicting client with enqueued lock
     statmany -s $MOUNT1/$tdir/f 1 500 &
     OPENPID=$!
     NOW=`date +%s`
-    do_facet mds sysctl -w lustre.fail_loc=0x8000030b  # hold enqueue
+    do_facet mds lctl set_param fail_loc=0x8000030b  # hold enqueue
     sleep 1
 #define OBD_FAIL_LDLM_BL_CALLBACK        0x305
-    do_facet client sysctl -w lustre.fail_loc=0x80000305  # drop cb, evict
+    do_facet client lctl set_param fail_loc=0x80000305  # drop cb, evict
     cancel_lru_locks mdc
     sleep 0.500s # wait to ensure first client is one that will be evicted
     openfile -f O_RDONLY $MOUNT2/$tdir/f0
@@ -433,8 +385,38 @@ test_19() { # Bug 10991 - resend of open request does not fail assertion.
 }
 run_test 19 "resend of open request"
 
+test_20() { #16389
+    BEFORE=`date +%s`
+    replay_barrier mds
+    touch $MOUNT1/a
+    touch $MOUNT2/b
+    umount $MOUNT2
+    facet_failover mds
+    df $MOUNT1 || return 1
+    rm $MOUNT1/a
+    zconf_mount `hostname` $MOUNT2 || error "mount $MOUNT2 fail"
+    TIER1=$((`date +%s` - BEFORE))
+    echo "First recovery time is $TIER1 sec"
+    BEFORE=`date +%s`
+    replay_barrier mds
+    touch $MOUNT1/a
+    touch $MOUNT2/b
+    umount $MOUNT2
+    facet_failover mds
+    df $MOUNT1 || return 1
+    rm $MOUNT1/a
+    zconf_mount `hostname` $MOUNT2 || error "mount $MOUNT2 fail"
+    TIER2=$((`date +%s` - BEFORE))
+    echo "Second recovery time is $TIER2 sec"
+    [ $TIER2 -ge $((TIER1 * 2)) ] && \
+        error "recovery time is growing $TIER2 > $TIER1"
+    return 0
+}
+run_test 20 "recovery time is not increasing"
+
 equals_msg `basename $0`: test complete, cleaning up
 SLEEP=$((`date +%s` - $NOW))
 [ $SLEEP -lt $TIMEOUT ] && sleep $SLEEP
+[ "$MOUNTED2" = yes ] && zconf_umount $HOSTNAME $MOUNT2 || true
 check_and_cleanup_lustre
-[ -f "$TESTSUITELOG" ] && cat $TESTSUITELOG || true
+[ -f "$TESTSUITELOG" ] && cat $TESTSUITELOG && grep -q FAIL $TESTSUITELOG && exit 1 || true
