@@ -67,7 +67,7 @@ OSTCOUNT=`lctl get_param -n lov.$LOVNAME.numobd`
 
 SHOW_QUOTA_USER="$LFS quota -v -u $TSTUSR $DIR"
 SHOW_QUOTA_GROUP="$LFS quota -v -g $TSTUSR $DIR"
-SHOW_QUOTA_INFO="$LFS quota -t $DIR"
+SHOW_QUOTA_INFO="$LFS quota -t -u $DIR; $LFS quota -t -g $DIR"
 
 # control the time of tests
 cycle=30
@@ -1143,9 +1143,18 @@ quota_set_version() {
 }
 
 # save quota version (both administrative and operational quotas)
+# the function will also switch to the new version and the new type
 quota_save_version() {
-        do_facet mgs "lctl conf_param ${FSNAME}-MDT*.mdt.quota_type=$1"
-        do_facet mgs "lctl conf_param ${FSNAME}-OST*.ost.quota_type=$1"
+    local spec=$1
+    local ver=$(tr -c -d "123" <<< $spec)
+    local type=$(tr -c -d "ug" <<< $spec)
+
+    $LFS quotaoff -ug $MOUNT # just in case
+    [ -n "$ver" ] && quota_set_version $ver
+    [ -n "$type" ] && { $LFS quotacheck -$type $MOUNT || error "quotacheck has failed"; }
+
+    do_facet mgs "lctl conf_param ${FSNAME}-MDT*.mdt.quota_type=$spec"
+    do_facet mgs "lctl conf_param ${FSNAME}-OST*.ost.quota_type=$spec"
 }
 
 test_14b(){
@@ -1239,9 +1248,8 @@ test_15(){
         [ $TOTAL_LIMIT -eq $LIMIT ] || error "  (group)total limits = $TOTAL_LIMIT; limit = $LIMIT, failed!"
         echo "  (group)total limits = $TOTAL_LIMIT; limit = $LIMIT, successful!"
         $LFS setquota -g $TSTUSR -b 0 -B 0 -i 0 -I 0 $DIR
-        $LFS quotaoff -ug $DIR
-        quota_set_version 1
-        $LFS quotacheck -ug $DIR || error "quotacheck failed"
+
+        quota_save_version "ug1"
 
         echo "Testing that >4GB quota limits fail on volume with quota v1"
         ! $LFS setquota -u $TSTUSR -b 0 -B $LIMIT -i 0 -I 0 $DIR
@@ -1721,9 +1729,6 @@ test_22() {
         local SAVEREFORMAT
 
         SAVEREFORMAT=$REFORMAT
-        $LFS quotaoff -ug $DIR || error "could not turn quotas off"
-        quota_set_version "1"
-        $LFS quotacheck -ug $DIR || error "quotacheck failed"
 
         quota_save_version "ug1"
 
@@ -1811,6 +1816,13 @@ test_24() {
 }
 run_test_with_stat 24 "test if lfs draws an asterix when limit is reached (16646) ==========="
 
+test_27() {
+        $LFS quota $TSTUSR $DIR && error "lfs succeeded with no type, but should have failed"
+        $LFS setquota $TSTUSR $DIR && error "lfs succeeded with no type, but should have failed"
+        return 0
+}
+run_test_with_stat 27 "lfs quota/setquota should handle wrong arguments (19612) ================="
+
 # turn off quota
 test_99()
 {
