@@ -93,7 +93,7 @@ init_test_env() {
     fi
     export MDSRATE=${MDSRATE:-"$LUSTRE/tests/mdsrate"}
     [ ! -f "$MDSRATE" ] && export MDSRATE=$(which mdsrate 2> /dev/null)
-    if ! echo $PATH | grep -q $LUSTRE/test/racer; then
+    if ! echo $PATH | grep -q $LUSTRE/tests/racer; then
         export PATH=$PATH:$LUSTRE/tests/racer
     fi
     export LCTL=${LCTL:-"$LUSTRE/utils/lctl"}
@@ -520,13 +520,14 @@ wait_update () {
         local RESULT
         local WAIT=0
         local sleep=5
-        while [ $WAIT -lt $MAX ]; do
+        while [ true ]; do
             sleep $sleep
             RESULT=$(do_node $node "$TEST")
             if [ $RESULT -eq $FINAL ]; then
                 echo "Updated after $WAIT sec: wanted $FINAL got $RESULT"
                 return 0
             fi
+            [ $WAIT -ge $MAX ] && break
             WAIT=$((WAIT + sleep))
             echo "Waiting $((MAX - WAIT)) secs for update"
         done
@@ -1590,7 +1591,7 @@ run_one() {
     local SAVE_UMASK=`umask`
     umask 0022
 
-    BEFORE=`date +%s`
+    local BEFORE=`date +%s`
     log "== test $testnum: $message ============ `date +%H:%M:%S` ($BEFORE)"
     #check_mds
     export TESTNAME=test_$testnum
@@ -1882,6 +1883,7 @@ calc_sum () {
 }
 
 calc_osc_kbytes () {
+        df $MOUNT > /dev/null
         $LCTL get_param -n osc.*[oO][sS][cC][-_]*.$1 | calc_sum
 }
 
@@ -1945,6 +1947,6 @@ mpi_run () {
 }
 
 mdsrate_cleanup () {
-    mpi_run -np $1 -machinefile $2 ${MDSRATE} --unlink --nfiles $3 --dir $4 --filefmt $5
+    mpi_run -np $1 -machinefile $2 ${MDSRATE} --unlink --nfiles $3 --dir $4 --filefmt $5 $6
 }
 
