@@ -1,30 +1,49 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  Copyright (C) 2002 Cluster File Systems, Inc.
- *   Author: Peter J. Braam <braam@clusterfs.com>
- *   Author: Phil Schwan <phil@clusterfs.com>
- *   Author: Robert Read <rread@clusterfs.com>
+ * GPL HEADER START
  *
- *   This file is part of Lustre, http://www.lustre.org.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   Lustre is free software; you can redistribute it and/or
- *   modify it under the terms of version 2 of the GNU General Public
- *   License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   Lustre is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   You should have received a copy of the GNU General Public License
- *   along with Lustre; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
  *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lustre/utils/liblustreapi.c
+ *
+ * Author: Peter J. Braam <braam@clusterfs.com>
+ * Author: Phil Schwan <phil@clusterfs.com>
+ * Author: Robert Read <rread@clusterfs.com>
  */
 
 /* for O_DIRECTORY */
+#ifndef _GNU_SOURCE
 #define _GNU_SOURCE
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -143,18 +162,23 @@ void llapi_printf(int level, char *fmt, ...)
         va_end(args);
 }
 
+/* size_units is unchanged if no specifier used */
 int parse_size(char *optarg, unsigned long long *size,
-               unsigned long long *size_units)
+               unsigned long long *size_units, int bytes_spec)
 {
         char *end;
 
-        *size = strtoul(optarg, &end, 0);
+        *size = strtoull(optarg, &end, 0);
 
         if (*end != '\0') {
                 if ((*end == 'b') && *(end+1) == '\0' &&
-                    (*size & (~0ULL << (64 - 9))) == 0) {
+                    (*size & (~0ULL << (64 - 9))) == 0 &&
+                    !bytes_spec) {
                         *size <<= 9;
                         *size_units = 1 << 9;
+                } else if ((*end == 'b') && *(end+1) == '\0' &&
+                           bytes_spec) {
+                        *size_units = 1;
                 } else if ((*end == 'k' || *end == 'K') &&
                            *(end+1) == '\0' && (*size &
                            (~0ULL << (64 - 10))) == 0) {
@@ -194,7 +218,7 @@ int parse_size(char *optarg, unsigned long long *size,
 }
 
 int llapi_file_open(const char *name, int flags, int mode,
-                    unsigned long stripe_size, int stripe_offset,
+                    unsigned long long stripe_size, int stripe_offset,
                     int stripe_count, int stripe_pattern)
 {
         struct lov_user_md lum = { 0 };
@@ -243,11 +267,12 @@ int llapi_file_open(const char *name, int flags, int mode,
                           stripe_count);
                 goto out;
         }
-        if (stripe_count > 0 && (__u64)stripe_size * stripe_count > 0xffffffff){
+
+        if (stripe_size >= (1ULL << 32)) {
                 errno = rc = -EINVAL;
-                llapi_err(LLAPI_MSG_ERROR, "error: stripe_size %lu * "
-                          "stripe_count %u exceeds 4GB", stripe_size, 
-                          stripe_count);
+                llapi_err_noerrno(LLAPI_MSG_ERROR,
+                                  "warning: stripe size larger than 4G "
+                                  "is not currently supported and would wrap");
                 goto out;
         }
 
@@ -277,7 +302,7 @@ out:
         return fd;
 }
 
-int llapi_file_create(const char *name, unsigned long stripe_size,
+int llapi_file_create(const char *name, unsigned long long stripe_size,
                       int stripe_offset, int stripe_count, int stripe_pattern)
 {
         int fd;
@@ -322,6 +347,40 @@ static void find_param_fini(struct find_param *param)
                 free(param->lmd);
 }
 
+int llapi_file_fget_lov_uuid(int fd, struct obd_uuid *lov_name)
+{
+        int rc = ioctl(fd, OBD_IOC_GETNAME, lov_name);
+        if (rc) {
+                rc = errno;
+                llapi_err(LLAPI_MSG_ERROR, "error: can't get lov name.");
+        }
+        return rc;
+}
+
+/* deprecated in favour of llapi_file_fget_lov_uuid() in 1.8 and 2.0 */
+int llapi_file_get_lov_fuuid(int fd, struct obd_uuid *lov_name)
+{
+        return llapi_file_fget_lov_uuid(fd, lov_name);
+}
+
+int llapi_file_get_lov_uuid(const char *path, struct obd_uuid *lov_uuid)
+{
+        int fd, rc;
+
+        fd = open(path, O_RDONLY);
+        if (fd < 0) {
+                rc = errno;
+                llapi_err(LLAPI_MSG_ERROR, "error opening %s\n", path);
+                return rc;
+        }
+
+        rc = llapi_file_fget_lov_uuid(fd, lov_uuid);
+
+        close(fd);
+
+        return rc;
+}
+
 /*
  * If uuidp is NULL, return the number of available obd uuids.
  * If uuidp is non-NULL, then it will return the uuids of the obds. If
@@ -330,22 +389,19 @@ static void find_param_fini(struct find_param *param)
  */
 int llapi_lov_get_uuids(int fd, struct obd_uuid *uuidp, int *ost_count)
 {
-        char lov_name[sizeof(struct obd_uuid)];
+        struct obd_uuid lov_name;
         char buf[1024];
         FILE *fp;
         int rc = 0, index = 0;
 
         /* Get the lov name */
-        rc = ioctl(fd, OBD_IOC_GETNAME, (void *) lov_name);
-        if (rc) {
-                rc = errno;
-                llapi_err(LLAPI_MSG_ERROR, "error: can't get lov name");
+        rc = llapi_file_fget_lov_uuid(fd, &lov_name);
+        if (rc)
                 return rc;
-        }
 
         /* Now get the ost uuids from /proc */
         snprintf(buf, sizeof(buf), "/proc/fs/lustre/lov/%s/target_obd",
-                 lov_name);
+                 lov_name.uuid);
         fp = fopen(buf, "r");
         if (fp == NULL) {
                 rc = errno;
@@ -374,13 +430,14 @@ int llapi_lov_get_uuids(int fd, struct obd_uuid *uuidp, int *ost_count)
  * returned in param->obdindex */
 static int setup_obd_uuid(DIR *dir, char *dname, struct find_param *param)
 {
+        struct obd_uuid lov_uuid;
         char uuid[sizeof(struct obd_uuid)];
         char buf[1024];
         FILE *fp;
         int rc = 0, index;
 
         /* Get the lov name */
-        rc = ioctl(dirfd(dir), OBD_IOC_GETNAME, (void *)uuid);
+        rc = llapi_file_fget_lov_uuid(dirfd(dir), &lov_uuid);
         if (rc) {
                 if (errno != ENOTTY) {
                         rc = errno;
@@ -396,7 +453,7 @@ static int setup_obd_uuid(DIR *dir, char *dname, struct find_param *param)
 
         /* Now get the ost uuids from /proc */
         snprintf(buf, sizeof(buf), "/proc/fs/lustre/lov/%s/target_obd",
-                 uuid);
+                 lov_uuid.uuid);
         fp = fopen(buf, "r");
         if (fp == NULL) {
                 rc = errno;
@@ -1064,6 +1121,26 @@ static int cb_find_init(char *path, DIR *parent, DIR *dir,
                 }
         }
 
+        if (param->check_uid) {
+                if (st->st_uid == param->uid) {
+                        if (param->exclude_uid)
+                                goto decided;
+                } else {
+                        if (!param->exclude_uid)
+                                goto decided;
+                }
+        }
+
+        if (param->check_gid) {
+                if (st->st_gid == param->gid) {
+                        if (param->exclude_gid)
+                                goto decided;
+                } else {
+                        if (!param->exclude_gid)
+                                goto decided;
+                }
+        }
+
         /* Check the time on mds. */
         if (!decision) {
                 int for_mds;
@@ -1396,7 +1473,7 @@ int llapi_ping(char *obd_type, char *obd_name)
 
         if (rc == 1)
                 return 0;
-        return rc;
+        return errno;
 }
 
 int llapi_target_iterate(int type_num, char **obd_type, void *args, llapi_cb_t cb)
@@ -1453,7 +1530,9 @@ static void do_target_check(char *obd_type_name, char *obd_name,
         int rc;
 
         rc = llapi_ping(obd_type_name, obd_name);
-        if (rc) {
+        if (rc == ENOTCONN || rc == ESHUTDOWN) {
+                llapi_printf(LLAPI_MSG_NORMAL, "%s inactive.\n", obd_name);
+        } else if (rc) {
                 llapi_err(LLAPI_MSG_ERROR, "error: check '%s'", obd_name);
         } else {
                 llapi_printf(LLAPI_MSG_NORMAL, "%s active.\n", obd_name);
