@@ -1,27 +1,44 @@
 /* -*- mode: c; c-basic-offset: 8; indent-tabs-mode: nil; -*-
  * vim:expandtab:shiftwidth=8:tabstop=8:
  *
- *  Copyright (C) 2002 Cluster File Systems, Inc.
- *   Author: Peter J. Braam <braam@clusterfs.com>
- *   Author: Phil Schwan <phil@clusterfs.com>
- *   Author: Andreas Dilger <adilger@clusterfs.com>
- *   Author: Robert Read <rread@clusterfs.com>
+ * GPL HEADER START
  *
- *   This file is part of Lustre, http://www.lustre.org.
+ * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
- *   Lustre is free software; you can redistribute it and/or
- *   modify it under the terms of version 2 of the GNU General Public
- *   License as published by the Free Software Foundation.
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License version 2 only,
+ * as published by the Free Software Foundation.
  *
- *   Lustre is distributed in the hope that it will be useful,
- *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License version 2 for more details (a copy is included
+ * in the LICENSE file that accompanied this code).
  *
- *   You should have received a copy of the GNU General Public License
- *   along with Lustre; if not, write to the Free Software
- *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU General Public License
+ * version 2 along with this program; If not, see
+ * http://www.sun.com/software/products/lustre/docs/GPLv2.pdf
  *
+ * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa Clara,
+ * CA 95054 USA or visit www.sun.com if you need additional information or
+ * have any questions.
+ *
+ * GPL HEADER END
+ */
+/*
+ * Copyright  2008 Sun Microsystems, Inc. All rights reserved
+ * Use is subject to license terms.
+ */
+/*
+ * This file is part of Lustre, http://www.lustre.org/
+ * Lustre is a trademark of Sun Microsystems, Inc.
+ *
+ * lustre/utils/obd.c
+ *
+ * Author: Peter J. Braam <braam@clusterfs.com>
+ * Author: Phil Schwan <phil@clusterfs.com>
+ * Author: Andreas Dilger <adilger@clusterfs.com>
+ * Author: Robert Read <rread@clusterfs.com>
  */
 
 #include <stdlib.h>
@@ -104,7 +121,7 @@ struct lsm_buffer {
         struct lov_oinfo *ptrs[MAX_STRIPES];
 } lsm_buffer;
 
-static int l2_ioctl(int dev_id, int opc, void *buf)
+static int l2_ioctl(int dev_id, unsigned int opc, void *buf)
 {
         return l_ioctl(dev_id, opc, buf);
 }
@@ -882,6 +899,51 @@ int jt_get_version(int argc, char **argv)
         return rc;
 }
 
+/*
+ * Print an obd device line with the ost_conn_uuid inserted, if the obd
+ * is an osc.
+ */
+static void print_obd_line(char *s)
+{
+        char buf[MAX_STRING_SIZE];
+        char obd_name[MAX_OBD_NAME];
+        FILE *fp = NULL;
+        char *ptr;
+
+        if (sscanf(s, " %*d %*s osc %s %*s %*d ", obd_name) == 0)
+                goto try_mdc;
+        snprintf(buf, sizeof(buf),
+                 "/proc/fs/lustre/osc/%s/ost_conn_uuid", obd_name);
+        if ((fp = fopen(buf, "r")) == NULL)
+                goto try_mdc;
+        goto got_one;
+
+try_mdc:
+        if (sscanf(s, " %*d %*s mdc %s %*s %*d ", obd_name) == 0)
+                goto fail;
+        snprintf(buf, sizeof(buf),
+                 "/proc/fs/lustre/mdc/%s/mds_conn_uuid", obd_name);
+        if ((fp = fopen(buf, "r")) == NULL)
+                goto fail;
+
+got_one:
+        fgets(buf, sizeof(buf), fp);
+        fclose(fp);
+
+        /* trim trailing newlines */
+        ptr = strrchr(buf, '\n');
+        if (ptr) *ptr = '\0';
+        ptr = strrchr(s, '\n');
+        if (ptr) *ptr = '\0';
+
+        printf("%s %s\n", s, buf);
+        return;
+
+fail:
+        printf("%s", s);
+        return;
+}
+
 /* get device list by ioctl */
 int jt_obd_list_ioctl(int argc, char **argv)
 {
@@ -889,7 +951,10 @@ int jt_obd_list_ioctl(int argc, char **argv)
         char buf[8192];
         struct obd_ioctl_data *data = (struct obd_ioctl_data *)buf;
 
-        if (argc != 1)
+        if (argc > 2)
+                return CMD_HELP;
+        /* Just ignore a -t option.  Only supported with /proc. */
+        else if (argc == 2 && strcmp(argv[1], "-t") != 0)
                 return CMD_HELP;
 
         for (index = 0;; index++) {
@@ -922,9 +987,16 @@ int jt_obd_list(int argc, char **argv)
         int rc;
         char buf[MAX_STRING_SIZE];
         FILE *fp = NULL;
+        int print_obd = 0;
 
-        if (argc != 1)
+        if (argc > 2)
                 return CMD_HELP;
+        else if (argc == 2) {
+                if (strcmp(argv[1], "-t") == 0)
+                        print_obd = 1;
+                else
+                        return CMD_HELP;
+        }
 
         fp = fopen(DEVICES_LIST, "r");
         if (fp == NULL) {
@@ -934,7 +1006,10 @@ int jt_obd_list(int argc, char **argv)
         }
 
         while (fgets(buf, sizeof(buf), fp) != NULL)
-                printf("%s", buf);
+                if (print_obd)
+                        print_obd_line(buf);
+                else
+                        printf("%s", buf);
 
         fclose(fp);
         return 0;
@@ -2224,7 +2299,7 @@ int jt_blockdev_info(int argc, char **argv)
         if (ino == 0ULL)
                 fprintf(stdout, "Not attached\n");
         else
-                fprintf(stdout, "attached to inode %llu\n", ino);
+                fprintf(stdout, "attached to inode "LPU64"\n", ino);
 out:
         close(fd);
         return -rc;
